# coding: utf-8

"""
    Onshape REST API

    The Onshape REST API consumed by all clients.  # noqa: E501

    OpenAPI spec version: 1.96
    Contact: api-support@onshape.zendesk.com
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class BTStandardContentUsageSummary(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'workspace_use_count': 'int',
        'top_used_components_by_workspaces': 'dict(str, BTComponentUsagesSummary)',
        'latest_version_use_count': 'int',
        'top_used_components_by_latest_versions': 'dict(str, BTComponentUsagesSummary)',
        'version_use_count': 'int',
        'top_components_used_along_with_by_latest_versions': 'dict(str, BTComponentUsagesSummary)',
        'top_users_by_workspaces': 'dict(str, BTUserUsagesSummary)',
        'top_users_by_latest_versions': 'dict(str, BTUserUsagesSummary)',
        'top_users_summary_done': 'bool'
    }

    attribute_map = {
        'workspace_use_count': 'workspaceUseCount',
        'top_used_components_by_workspaces': 'topUsedComponentsByWorkspaces',
        'latest_version_use_count': 'latestVersionUseCount',
        'top_used_components_by_latest_versions': 'topUsedComponentsByLatestVersions',
        'version_use_count': 'versionUseCount',
        'top_components_used_along_with_by_latest_versions': 'topComponentsUsedAlongWithByLatestVersions',
        'top_users_by_workspaces': 'topUsersByWorkspaces',
        'top_users_by_latest_versions': 'topUsersByLatestVersions',
        'top_users_summary_done': 'topUsersSummaryDone'
    }

    def __init__(self, workspace_use_count=None, top_used_components_by_workspaces=None, latest_version_use_count=None, top_used_components_by_latest_versions=None, version_use_count=None, top_components_used_along_with_by_latest_versions=None, top_users_by_workspaces=None, top_users_by_latest_versions=None, top_users_summary_done=None):  # noqa: E501
        """BTStandardContentUsageSummary - a model defined in OpenAPI"""  # noqa: E501

        self._workspace_use_count = None
        self._top_used_components_by_workspaces = None
        self._latest_version_use_count = None
        self._top_used_components_by_latest_versions = None
        self._version_use_count = None
        self._top_components_used_along_with_by_latest_versions = None
        self._top_users_by_workspaces = None
        self._top_users_by_latest_versions = None
        self._top_users_summary_done = None
        self.discriminator = None

        if workspace_use_count is not None:
            self.workspace_use_count = workspace_use_count
        if top_used_components_by_workspaces is not None:
            self.top_used_components_by_workspaces = top_used_components_by_workspaces
        if latest_version_use_count is not None:
            self.latest_version_use_count = latest_version_use_count
        if top_used_components_by_latest_versions is not None:
            self.top_used_components_by_latest_versions = top_used_components_by_latest_versions
        if version_use_count is not None:
            self.version_use_count = version_use_count
        if top_components_used_along_with_by_latest_versions is not None:
            self.top_components_used_along_with_by_latest_versions = top_components_used_along_with_by_latest_versions
        if top_users_by_workspaces is not None:
            self.top_users_by_workspaces = top_users_by_workspaces
        if top_users_by_latest_versions is not None:
            self.top_users_by_latest_versions = top_users_by_latest_versions
        if top_users_summary_done is not None:
            self.top_users_summary_done = top_users_summary_done

    @property
    def workspace_use_count(self):
        """Gets the workspace_use_count of this BTStandardContentUsageSummary.  # noqa: E501


        :return: The workspace_use_count of this BTStandardContentUsageSummary.  # noqa: E501
        :rtype: int
        """
        return self._workspace_use_count

    @workspace_use_count.setter
    def workspace_use_count(self, workspace_use_count):
        """Sets the workspace_use_count of this BTStandardContentUsageSummary.


        :param workspace_use_count: The workspace_use_count of this BTStandardContentUsageSummary.  # noqa: E501
        :type: int
        """

        self._workspace_use_count = workspace_use_count

    @property
    def top_used_components_by_workspaces(self):
        """Gets the top_used_components_by_workspaces of this BTStandardContentUsageSummary.  # noqa: E501


        :return: The top_used_components_by_workspaces of this BTStandardContentUsageSummary.  # noqa: E501
        :rtype: dict(str, BTComponentUsagesSummary)
        """
        return self._top_used_components_by_workspaces

    @top_used_components_by_workspaces.setter
    def top_used_components_by_workspaces(self, top_used_components_by_workspaces):
        """Sets the top_used_components_by_workspaces of this BTStandardContentUsageSummary.


        :param top_used_components_by_workspaces: The top_used_components_by_workspaces of this BTStandardContentUsageSummary.  # noqa: E501
        :type: dict(str, BTComponentUsagesSummary)
        """

        self._top_used_components_by_workspaces = top_used_components_by_workspaces

    @property
    def latest_version_use_count(self):
        """Gets the latest_version_use_count of this BTStandardContentUsageSummary.  # noqa: E501


        :return: The latest_version_use_count of this BTStandardContentUsageSummary.  # noqa: E501
        :rtype: int
        """
        return self._latest_version_use_count

    @latest_version_use_count.setter
    def latest_version_use_count(self, latest_version_use_count):
        """Sets the latest_version_use_count of this BTStandardContentUsageSummary.


        :param latest_version_use_count: The latest_version_use_count of this BTStandardContentUsageSummary.  # noqa: E501
        :type: int
        """

        self._latest_version_use_count = latest_version_use_count

    @property
    def top_used_components_by_latest_versions(self):
        """Gets the top_used_components_by_latest_versions of this BTStandardContentUsageSummary.  # noqa: E501


        :return: The top_used_components_by_latest_versions of this BTStandardContentUsageSummary.  # noqa: E501
        :rtype: dict(str, BTComponentUsagesSummary)
        """
        return self._top_used_components_by_latest_versions

    @top_used_components_by_latest_versions.setter
    def top_used_components_by_latest_versions(self, top_used_components_by_latest_versions):
        """Sets the top_used_components_by_latest_versions of this BTStandardContentUsageSummary.


        :param top_used_components_by_latest_versions: The top_used_components_by_latest_versions of this BTStandardContentUsageSummary.  # noqa: E501
        :type: dict(str, BTComponentUsagesSummary)
        """

        self._top_used_components_by_latest_versions = top_used_components_by_latest_versions

    @property
    def version_use_count(self):
        """Gets the version_use_count of this BTStandardContentUsageSummary.  # noqa: E501


        :return: The version_use_count of this BTStandardContentUsageSummary.  # noqa: E501
        :rtype: int
        """
        return self._version_use_count

    @version_use_count.setter
    def version_use_count(self, version_use_count):
        """Sets the version_use_count of this BTStandardContentUsageSummary.


        :param version_use_count: The version_use_count of this BTStandardContentUsageSummary.  # noqa: E501
        :type: int
        """

        self._version_use_count = version_use_count

    @property
    def top_components_used_along_with_by_latest_versions(self):
        """Gets the top_components_used_along_with_by_latest_versions of this BTStandardContentUsageSummary.  # noqa: E501


        :return: The top_components_used_along_with_by_latest_versions of this BTStandardContentUsageSummary.  # noqa: E501
        :rtype: dict(str, BTComponentUsagesSummary)
        """
        return self._top_components_used_along_with_by_latest_versions

    @top_components_used_along_with_by_latest_versions.setter
    def top_components_used_along_with_by_latest_versions(self, top_components_used_along_with_by_latest_versions):
        """Sets the top_components_used_along_with_by_latest_versions of this BTStandardContentUsageSummary.


        :param top_components_used_along_with_by_latest_versions: The top_components_used_along_with_by_latest_versions of this BTStandardContentUsageSummary.  # noqa: E501
        :type: dict(str, BTComponentUsagesSummary)
        """

        self._top_components_used_along_with_by_latest_versions = top_components_used_along_with_by_latest_versions

    @property
    def top_users_by_workspaces(self):
        """Gets the top_users_by_workspaces of this BTStandardContentUsageSummary.  # noqa: E501


        :return: The top_users_by_workspaces of this BTStandardContentUsageSummary.  # noqa: E501
        :rtype: dict(str, BTUserUsagesSummary)
        """
        return self._top_users_by_workspaces

    @top_users_by_workspaces.setter
    def top_users_by_workspaces(self, top_users_by_workspaces):
        """Sets the top_users_by_workspaces of this BTStandardContentUsageSummary.


        :param top_users_by_workspaces: The top_users_by_workspaces of this BTStandardContentUsageSummary.  # noqa: E501
        :type: dict(str, BTUserUsagesSummary)
        """

        self._top_users_by_workspaces = top_users_by_workspaces

    @property
    def top_users_by_latest_versions(self):
        """Gets the top_users_by_latest_versions of this BTStandardContentUsageSummary.  # noqa: E501


        :return: The top_users_by_latest_versions of this BTStandardContentUsageSummary.  # noqa: E501
        :rtype: dict(str, BTUserUsagesSummary)
        """
        return self._top_users_by_latest_versions

    @top_users_by_latest_versions.setter
    def top_users_by_latest_versions(self, top_users_by_latest_versions):
        """Sets the top_users_by_latest_versions of this BTStandardContentUsageSummary.


        :param top_users_by_latest_versions: The top_users_by_latest_versions of this BTStandardContentUsageSummary.  # noqa: E501
        :type: dict(str, BTUserUsagesSummary)
        """

        self._top_users_by_latest_versions = top_users_by_latest_versions

    @property
    def top_users_summary_done(self):
        """Gets the top_users_summary_done of this BTStandardContentUsageSummary.  # noqa: E501


        :return: The top_users_summary_done of this BTStandardContentUsageSummary.  # noqa: E501
        :rtype: bool
        """
        return self._top_users_summary_done

    @top_users_summary_done.setter
    def top_users_summary_done(self, top_users_summary_done):
        """Sets the top_users_summary_done of this BTStandardContentUsageSummary.


        :param top_users_summary_done: The top_users_summary_done of this BTStandardContentUsageSummary.  # noqa: E501
        :type: bool
        """

        self._top_users_summary_done = top_users_summary_done

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, BTStandardContentUsageSummary):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Onshape REST API

    The Onshape REST API consumed by all clients.  # noqa: E501

    OpenAPI spec version: 1.96
    Contact: api-support@onshape.zendesk.com
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class BTWorkspacePartParams(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'name': 'str',
        'description': 'str',
        'project': 'str',
        'vendor': 'str',
        'title1': 'str',
        'revision': 'str',
        'title2': 'str',
        'title3': 'str',
        'product_line': 'str',
        'configuration': 'str',
        'element_id': 'str',
        'part_number': 'str',
        'part_id': 'str',
        'appearance': 'BTPartAppearanceParams',
        'connection_id': 'str',
        'material': 'BTMaterialParams',
        'custom_property_definitions': 'list[BTCustomPropertyDefinitionParams]',
        'apply_update_to_all_configurations': 'bool',
        'custom_properties': 'list[BTNameValuePair]'
    }

    attribute_map = {
        'name': 'name',
        'description': 'description',
        'project': 'project',
        'vendor': 'vendor',
        'title1': 'title1',
        'revision': 'revision',
        'title2': 'title2',
        'title3': 'title3',
        'product_line': 'productLine',
        'configuration': 'configuration',
        'element_id': 'elementId',
        'part_number': 'partNumber',
        'part_id': 'partId',
        'appearance': 'appearance',
        'connection_id': 'connectionId',
        'material': 'material',
        'custom_property_definitions': 'customPropertyDefinitions',
        'apply_update_to_all_configurations': 'applyUpdateToAllConfigurations',
        'custom_properties': 'customProperties'
    }

    def __init__(self, name=None, description=None, project=None, vendor=None, title1=None, revision=None, title2=None, title3=None, product_line=None, configuration=None, element_id=None, part_number=None, part_id=None, appearance=None, connection_id=None, material=None, custom_property_definitions=None, apply_update_to_all_configurations=None, custom_properties=None):  # noqa: E501
        """BTWorkspacePartParams - a model defined in OpenAPI"""  # noqa: E501

        self._name = None
        self._description = None
        self._project = None
        self._vendor = None
        self._title1 = None
        self._revision = None
        self._title2 = None
        self._title3 = None
        self._product_line = None
        self._configuration = None
        self._element_id = None
        self._part_number = None
        self._part_id = None
        self._appearance = None
        self._connection_id = None
        self._material = None
        self._custom_property_definitions = None
        self._apply_update_to_all_configurations = None
        self._custom_properties = None
        self.discriminator = None

        if name is not None:
            self.name = name
        if description is not None:
            self.description = description
        if project is not None:
            self.project = project
        if vendor is not None:
            self.vendor = vendor
        if title1 is not None:
            self.title1 = title1
        if revision is not None:
            self.revision = revision
        if title2 is not None:
            self.title2 = title2
        if title3 is not None:
            self.title3 = title3
        if product_line is not None:
            self.product_line = product_line
        if configuration is not None:
            self.configuration = configuration
        if element_id is not None:
            self.element_id = element_id
        if part_number is not None:
            self.part_number = part_number
        if part_id is not None:
            self.part_id = part_id
        if appearance is not None:
            self.appearance = appearance
        if connection_id is not None:
            self.connection_id = connection_id
        if material is not None:
            self.material = material
        if custom_property_definitions is not None:
            self.custom_property_definitions = custom_property_definitions
        if apply_update_to_all_configurations is not None:
            self.apply_update_to_all_configurations = apply_update_to_all_configurations
        if custom_properties is not None:
            self.custom_properties = custom_properties

    @property
    def name(self):
        """Gets the name of this BTWorkspacePartParams.  # noqa: E501


        :return: The name of this BTWorkspacePartParams.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this BTWorkspacePartParams.


        :param name: The name of this BTWorkspacePartParams.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def description(self):
        """Gets the description of this BTWorkspacePartParams.  # noqa: E501


        :return: The description of this BTWorkspacePartParams.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this BTWorkspacePartParams.


        :param description: The description of this BTWorkspacePartParams.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def project(self):
        """Gets the project of this BTWorkspacePartParams.  # noqa: E501


        :return: The project of this BTWorkspacePartParams.  # noqa: E501
        :rtype: str
        """
        return self._project

    @project.setter
    def project(self, project):
        """Sets the project of this BTWorkspacePartParams.


        :param project: The project of this BTWorkspacePartParams.  # noqa: E501
        :type: str
        """

        self._project = project

    @property
    def vendor(self):
        """Gets the vendor of this BTWorkspacePartParams.  # noqa: E501


        :return: The vendor of this BTWorkspacePartParams.  # noqa: E501
        :rtype: str
        """
        return self._vendor

    @vendor.setter
    def vendor(self, vendor):
        """Sets the vendor of this BTWorkspacePartParams.


        :param vendor: The vendor of this BTWorkspacePartParams.  # noqa: E501
        :type: str
        """

        self._vendor = vendor

    @property
    def title1(self):
        """Gets the title1 of this BTWorkspacePartParams.  # noqa: E501


        :return: The title1 of this BTWorkspacePartParams.  # noqa: E501
        :rtype: str
        """
        return self._title1

    @title1.setter
    def title1(self, title1):
        """Sets the title1 of this BTWorkspacePartParams.


        :param title1: The title1 of this BTWorkspacePartParams.  # noqa: E501
        :type: str
        """

        self._title1 = title1

    @property
    def revision(self):
        """Gets the revision of this BTWorkspacePartParams.  # noqa: E501


        :return: The revision of this BTWorkspacePartParams.  # noqa: E501
        :rtype: str
        """
        return self._revision

    @revision.setter
    def revision(self, revision):
        """Sets the revision of this BTWorkspacePartParams.


        :param revision: The revision of this BTWorkspacePartParams.  # noqa: E501
        :type: str
        """

        self._revision = revision

    @property
    def title2(self):
        """Gets the title2 of this BTWorkspacePartParams.  # noqa: E501


        :return: The title2 of this BTWorkspacePartParams.  # noqa: E501
        :rtype: str
        """
        return self._title2

    @title2.setter
    def title2(self, title2):
        """Sets the title2 of this BTWorkspacePartParams.


        :param title2: The title2 of this BTWorkspacePartParams.  # noqa: E501
        :type: str
        """

        self._title2 = title2

    @property
    def title3(self):
        """Gets the title3 of this BTWorkspacePartParams.  # noqa: E501


        :return: The title3 of this BTWorkspacePartParams.  # noqa: E501
        :rtype: str
        """
        return self._title3

    @title3.setter
    def title3(self, title3):
        """Sets the title3 of this BTWorkspacePartParams.


        :param title3: The title3 of this BTWorkspacePartParams.  # noqa: E501
        :type: str
        """

        self._title3 = title3

    @property
    def product_line(self):
        """Gets the product_line of this BTWorkspacePartParams.  # noqa: E501


        :return: The product_line of this BTWorkspacePartParams.  # noqa: E501
        :rtype: str
        """
        return self._product_line

    @product_line.setter
    def product_line(self, product_line):
        """Sets the product_line of this BTWorkspacePartParams.


        :param product_line: The product_line of this BTWorkspacePartParams.  # noqa: E501
        :type: str
        """

        self._product_line = product_line

    @property
    def configuration(self):
        """Gets the configuration of this BTWorkspacePartParams.  # noqa: E501


        :return: The configuration of this BTWorkspacePartParams.  # noqa: E501
        :rtype: str
        """
        return self._configuration

    @configuration.setter
    def configuration(self, configuration):
        """Sets the configuration of this BTWorkspacePartParams.


        :param configuration: The configuration of this BTWorkspacePartParams.  # noqa: E501
        :type: str
        """

        self._configuration = configuration

    @property
    def element_id(self):
        """Gets the element_id of this BTWorkspacePartParams.  # noqa: E501


        :return: The element_id of this BTWorkspacePartParams.  # noqa: E501
        :rtype: str
        """
        return self._element_id

    @element_id.setter
    def element_id(self, element_id):
        """Sets the element_id of this BTWorkspacePartParams.


        :param element_id: The element_id of this BTWorkspacePartParams.  # noqa: E501
        :type: str
        """

        self._element_id = element_id

    @property
    def part_number(self):
        """Gets the part_number of this BTWorkspacePartParams.  # noqa: E501


        :return: The part_number of this BTWorkspacePartParams.  # noqa: E501
        :rtype: str
        """
        return self._part_number

    @part_number.setter
    def part_number(self, part_number):
        """Sets the part_number of this BTWorkspacePartParams.


        :param part_number: The part_number of this BTWorkspacePartParams.  # noqa: E501
        :type: str
        """

        self._part_number = part_number

    @property
    def part_id(self):
        """Gets the part_id of this BTWorkspacePartParams.  # noqa: E501


        :return: The part_id of this BTWorkspacePartParams.  # noqa: E501
        :rtype: str
        """
        return self._part_id

    @part_id.setter
    def part_id(self, part_id):
        """Sets the part_id of this BTWorkspacePartParams.


        :param part_id: The part_id of this BTWorkspacePartParams.  # noqa: E501
        :type: str
        """

        self._part_id = part_id

    @property
    def appearance(self):
        """Gets the appearance of this BTWorkspacePartParams.  # noqa: E501


        :return: The appearance of this BTWorkspacePartParams.  # noqa: E501
        :rtype: BTPartAppearanceParams
        """
        return self._appearance

    @appearance.setter
    def appearance(self, appearance):
        """Sets the appearance of this BTWorkspacePartParams.


        :param appearance: The appearance of this BTWorkspacePartParams.  # noqa: E501
        :type: BTPartAppearanceParams
        """

        self._appearance = appearance

    @property
    def connection_id(self):
        """Gets the connection_id of this BTWorkspacePartParams.  # noqa: E501


        :return: The connection_id of this BTWorkspacePartParams.  # noqa: E501
        :rtype: str
        """
        return self._connection_id

    @connection_id.setter
    def connection_id(self, connection_id):
        """Sets the connection_id of this BTWorkspacePartParams.


        :param connection_id: The connection_id of this BTWorkspacePartParams.  # noqa: E501
        :type: str
        """

        self._connection_id = connection_id

    @property
    def material(self):
        """Gets the material of this BTWorkspacePartParams.  # noqa: E501


        :return: The material of this BTWorkspacePartParams.  # noqa: E501
        :rtype: BTMaterialParams
        """
        return self._material

    @material.setter
    def material(self, material):
        """Sets the material of this BTWorkspacePartParams.


        :param material: The material of this BTWorkspacePartParams.  # noqa: E501
        :type: BTMaterialParams
        """

        self._material = material

    @property
    def custom_property_definitions(self):
        """Gets the custom_property_definitions of this BTWorkspacePartParams.  # noqa: E501


        :return: The custom_property_definitions of this BTWorkspacePartParams.  # noqa: E501
        :rtype: list[BTCustomPropertyDefinitionParams]
        """
        return self._custom_property_definitions

    @custom_property_definitions.setter
    def custom_property_definitions(self, custom_property_definitions):
        """Sets the custom_property_definitions of this BTWorkspacePartParams.


        :param custom_property_definitions: The custom_property_definitions of this BTWorkspacePartParams.  # noqa: E501
        :type: list[BTCustomPropertyDefinitionParams]
        """

        self._custom_property_definitions = custom_property_definitions

    @property
    def apply_update_to_all_configurations(self):
        """Gets the apply_update_to_all_configurations of this BTWorkspacePartParams.  # noqa: E501


        :return: The apply_update_to_all_configurations of this BTWorkspacePartParams.  # noqa: E501
        :rtype: bool
        """
        return self._apply_update_to_all_configurations

    @apply_update_to_all_configurations.setter
    def apply_update_to_all_configurations(self, apply_update_to_all_configurations):
        """Sets the apply_update_to_all_configurations of this BTWorkspacePartParams.


        :param apply_update_to_all_configurations: The apply_update_to_all_configurations of this BTWorkspacePartParams.  # noqa: E501
        :type: bool
        """

        self._apply_update_to_all_configurations = apply_update_to_all_configurations

    @property
    def custom_properties(self):
        """Gets the custom_properties of this BTWorkspacePartParams.  # noqa: E501


        :return: The custom_properties of this BTWorkspacePartParams.  # noqa: E501
        :rtype: list[BTNameValuePair]
        """
        return self._custom_properties

    @custom_properties.setter
    def custom_properties(self, custom_properties):
        """Sets the custom_properties of this BTWorkspacePartParams.


        :param custom_properties: The custom_properties of this BTWorkspacePartParams.  # noqa: E501
        :type: list[BTNameValuePair]
        """

        self._custom_properties = custom_properties

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, BTWorkspacePartParams):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Onshape REST API

    The Onshape REST API consumed by all clients.  # noqa: E501

    OpenAPI spec version: 1.96
    Contact: api-support@onshape.zendesk.com
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class BTDBViewerCollectionConfig(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'collection': 'str',
        'db': 'str',
        'query_fields': 'list[str]',
        'hidden_fields': 'list[str]',
        'binary_fields': 'list[str]',
        'has_created_date': 'bool'
    }

    attribute_map = {
        'collection': 'collection',
        'db': 'db',
        'query_fields': 'queryFields',
        'hidden_fields': 'hiddenFields',
        'binary_fields': 'binaryFields',
        'has_created_date': 'hasCreatedDate'
    }

    def __init__(self, collection=None, db=None, query_fields=None, hidden_fields=None, binary_fields=None, has_created_date=None):  # noqa: E501
        """BTDBViewerCollectionConfig - a model defined in OpenAPI"""  # noqa: E501

        self._collection = None
        self._db = None
        self._query_fields = None
        self._hidden_fields = None
        self._binary_fields = None
        self._has_created_date = None
        self.discriminator = None

        if collection is not None:
            self.collection = collection
        if db is not None:
            self.db = db
        if query_fields is not None:
            self.query_fields = query_fields
        if hidden_fields is not None:
            self.hidden_fields = hidden_fields
        if binary_fields is not None:
            self.binary_fields = binary_fields
        if has_created_date is not None:
            self.has_created_date = has_created_date

    @property
    def collection(self):
        """Gets the collection of this BTDBViewerCollectionConfig.  # noqa: E501


        :return: The collection of this BTDBViewerCollectionConfig.  # noqa: E501
        :rtype: str
        """
        return self._collection

    @collection.setter
    def collection(self, collection):
        """Sets the collection of this BTDBViewerCollectionConfig.


        :param collection: The collection of this BTDBViewerCollectionConfig.  # noqa: E501
        :type: str
        """

        self._collection = collection

    @property
    def db(self):
        """Gets the db of this BTDBViewerCollectionConfig.  # noqa: E501


        :return: The db of this BTDBViewerCollectionConfig.  # noqa: E501
        :rtype: str
        """
        return self._db

    @db.setter
    def db(self, db):
        """Sets the db of this BTDBViewerCollectionConfig.


        :param db: The db of this BTDBViewerCollectionConfig.  # noqa: E501
        :type: str
        """

        self._db = db

    @property
    def query_fields(self):
        """Gets the query_fields of this BTDBViewerCollectionConfig.  # noqa: E501


        :return: The query_fields of this BTDBViewerCollectionConfig.  # noqa: E501
        :rtype: list[str]
        """
        return self._query_fields

    @query_fields.setter
    def query_fields(self, query_fields):
        """Sets the query_fields of this BTDBViewerCollectionConfig.


        :param query_fields: The query_fields of this BTDBViewerCollectionConfig.  # noqa: E501
        :type: list[str]
        """

        self._query_fields = query_fields

    @property
    def hidden_fields(self):
        """Gets the hidden_fields of this BTDBViewerCollectionConfig.  # noqa: E501


        :return: The hidden_fields of this BTDBViewerCollectionConfig.  # noqa: E501
        :rtype: list[str]
        """
        return self._hidden_fields

    @hidden_fields.setter
    def hidden_fields(self, hidden_fields):
        """Sets the hidden_fields of this BTDBViewerCollectionConfig.


        :param hidden_fields: The hidden_fields of this BTDBViewerCollectionConfig.  # noqa: E501
        :type: list[str]
        """

        self._hidden_fields = hidden_fields

    @property
    def binary_fields(self):
        """Gets the binary_fields of this BTDBViewerCollectionConfig.  # noqa: E501


        :return: The binary_fields of this BTDBViewerCollectionConfig.  # noqa: E501
        :rtype: list[str]
        """
        return self._binary_fields

    @binary_fields.setter
    def binary_fields(self, binary_fields):
        """Sets the binary_fields of this BTDBViewerCollectionConfig.


        :param binary_fields: The binary_fields of this BTDBViewerCollectionConfig.  # noqa: E501
        :type: list[str]
        """

        self._binary_fields = binary_fields

    @property
    def has_created_date(self):
        """Gets the has_created_date of this BTDBViewerCollectionConfig.  # noqa: E501


        :return: The has_created_date of this BTDBViewerCollectionConfig.  # noqa: E501
        :rtype: bool
        """
        return self._has_created_date

    @has_created_date.setter
    def has_created_date(self, has_created_date):
        """Sets the has_created_date of this BTDBViewerCollectionConfig.


        :param has_created_date: The has_created_date of this BTDBViewerCollectionConfig.  # noqa: E501
        :type: bool
        """

        self._has_created_date = has_created_date

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, BTDBViewerCollectionConfig):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Onshape REST API

    The Onshape REST API consumed by all clients.  # noqa: E501

    OpenAPI spec version: 1.96
    Contact: api-support@onshape.zendesk.com
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class SecurityScheme(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'type': 'str',
        'description': 'str',
        'name': 'str',
        'getref': 'str',
        '_in': 'str',
        'scheme': 'str',
        'bearer_format': 'str',
        'flows': 'OAuthFlows',
        'open_id_connect_url': 'str',
        'extensions': 'dict(str, object)'
    }

    attribute_map = {
        'type': 'type',
        'description': 'description',
        'name': 'name',
        'getref': 'get$ref',
        '_in': 'in',
        'scheme': 'scheme',
        'bearer_format': 'bearerFormat',
        'flows': 'flows',
        'open_id_connect_url': 'openIdConnectUrl',
        'extensions': 'extensions'
    }

    def __init__(self, type=None, description=None, name=None, getref=None, _in=None, scheme=None, bearer_format=None, flows=None, open_id_connect_url=None, extensions=None):  # noqa: E501
        """SecurityScheme - a model defined in OpenAPI"""  # noqa: E501

        self._type = None
        self._description = None
        self._name = None
        self._getref = None
        self.__in = None
        self._scheme = None
        self._bearer_format = None
        self._flows = None
        self._open_id_connect_url = None
        self._extensions = None
        self.discriminator = None

        if type is not None:
            self.type = type
        if description is not None:
            self.description = description
        if name is not None:
            self.name = name
        if getref is not None:
            self.getref = getref
        if _in is not None:
            self._in = _in
        if scheme is not None:
            self.scheme = scheme
        if bearer_format is not None:
            self.bearer_format = bearer_format
        if flows is not None:
            self.flows = flows
        if open_id_connect_url is not None:
            self.open_id_connect_url = open_id_connect_url
        if extensions is not None:
            self.extensions = extensions

    @property
    def type(self):
        """Gets the type of this SecurityScheme.  # noqa: E501


        :return: The type of this SecurityScheme.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this SecurityScheme.


        :param type: The type of this SecurityScheme.  # noqa: E501
        :type: str
        """
        allowed_values = ["apiKey", "http", "oauth2", "openIdConnect"]  # noqa: E501
        if type not in allowed_values:
            raise ValueError(
                "Invalid value for `type` ({0}), must be one of {1}"  # noqa: E501
                .format(type, allowed_values)
            )

        self._type = type

    @property
    def description(self):
        """Gets the description of this SecurityScheme.  # noqa: E501


        :return: The description of this SecurityScheme.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this SecurityScheme.


        :param description: The description of this SecurityScheme.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def name(self):
        """Gets the name of this SecurityScheme.  # noqa: E501


        :return: The name of this SecurityScheme.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this SecurityScheme.


        :param name: The name of this SecurityScheme.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def getref(self):
        """Gets the getref of this SecurityScheme.  # noqa: E501


        :return: The getref of this SecurityScheme.  # noqa: E501
        :rtype: str
        """
        return self._getref

    @getref.setter
    def getref(self, getref):
        """Sets the getref of this SecurityScheme.


        :param getref: The getref of this SecurityScheme.  # noqa: E501
        :type: str
        """

        self._getref = getref

    @property
    def _in(self):
        """Gets the _in of this SecurityScheme.  # noqa: E501


        :return: The _in of this SecurityScheme.  # noqa: E501
        :rtype: str
        """
        return self.__in

    @_in.setter
    def _in(self, _in):
        """Sets the _in of this SecurityScheme.


        :param _in: The _in of this SecurityScheme.  # noqa: E501
        :type: str
        """
        allowed_values = ["cookie", "header", "query"]  # noqa: E501
        if _in not in allowed_values:
            raise ValueError(
                "Invalid value for `_in` ({0}), must be one of {1}"  # noqa: E501
                .format(_in, allowed_values)
            )

        self.__in = _in

    @property
    def scheme(self):
        """Gets the scheme of this SecurityScheme.  # noqa: E501


        :return: The scheme of this SecurityScheme.  # noqa: E501
        :rtype: str
        """
        return self._scheme

    @scheme.setter
    def scheme(self, scheme):
        """Sets the scheme of this SecurityScheme.


        :param scheme: The scheme of this SecurityScheme.  # noqa: E501
        :type: str
        """

        self._scheme = scheme

    @property
    def bearer_format(self):
        """Gets the bearer_format of this SecurityScheme.  # noqa: E501


        :return: The bearer_format of this SecurityScheme.  # noqa: E501
        :rtype: str
        """
        return self._bearer_format

    @bearer_format.setter
    def bearer_format(self, bearer_format):
        """Sets the bearer_format of this SecurityScheme.


        :param bearer_format: The bearer_format of this SecurityScheme.  # noqa: E501
        :type: str
        """

        self._bearer_format = bearer_format

    @property
    def flows(self):
        """Gets the flows of this SecurityScheme.  # noqa: E501


        :return: The flows of this SecurityScheme.  # noqa: E501
        :rtype: OAuthFlows
        """
        return self._flows

    @flows.setter
    def flows(self, flows):
        """Sets the flows of this SecurityScheme.


        :param flows: The flows of this SecurityScheme.  # noqa: E501
        :type: OAuthFlows
        """

        self._flows = flows

    @property
    def open_id_connect_url(self):
        """Gets the open_id_connect_url of this SecurityScheme.  # noqa: E501


        :return: The open_id_connect_url of this SecurityScheme.  # noqa: E501
        :rtype: str
        """
        return self._open_id_connect_url

    @open_id_connect_url.setter
    def open_id_connect_url(self, open_id_connect_url):
        """Sets the open_id_connect_url of this SecurityScheme.


        :param open_id_connect_url: The open_id_connect_url of this SecurityScheme.  # noqa: E501
        :type: str
        """

        self._open_id_connect_url = open_id_connect_url

    @property
    def extensions(self):
        """Gets the extensions of this SecurityScheme.  # noqa: E501


        :return: The extensions of this SecurityScheme.  # noqa: E501
        :rtype: dict(str, object)
        """
        return self._extensions

    @extensions.setter
    def extensions(self, extensions):
        """Sets the extensions of this SecurityScheme.


        :param extensions: The extensions of this SecurityScheme.  # noqa: E501
        :type: dict(str, object)
        """

        self._extensions = extensions

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SecurityScheme):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Onshape REST API

    The Onshape REST API consumed by all clients.  # noqa: E501

    OpenAPI spec version: 1.96
    Contact: api-support@onshape.zendesk.com
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class BTWebhookParams(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'id': 'str',
        'options': 'BTWebhookOptions',
        'filter': 'str',
        'user_id': 'str',
        'data': 'str',
        'company_id': 'str',
        'client_id': 'str',
        'workspace_id': 'str',
        'element_id': 'str',
        'part_id': 'str',
        'project_id': 'str',
        'folder_id': 'str',
        'version_id': 'str',
        'document_id': 'str',
        'url': 'str',
        'events': 'list[str]'
    }

    attribute_map = {
        'id': 'id',
        'options': 'options',
        'filter': 'filter',
        'user_id': 'userId',
        'data': 'data',
        'company_id': 'companyId',
        'client_id': 'clientId',
        'workspace_id': 'workspaceId',
        'element_id': 'elementId',
        'part_id': 'partId',
        'project_id': 'projectId',
        'folder_id': 'folderId',
        'version_id': 'versionId',
        'document_id': 'documentId',
        'url': 'url',
        'events': 'events'
    }

    def __init__(self, id=None, options=None, filter=None, user_id=None, data=None, company_id=None, client_id=None, workspace_id=None, element_id=None, part_id=None, project_id=None, folder_id=None, version_id=None, document_id=None, url=None, events=None):  # noqa: E501
        """BTWebhookParams - a model defined in OpenAPI"""  # noqa: E501

        self._id = None
        self._options = None
        self._filter = None
        self._user_id = None
        self._data = None
        self._company_id = None
        self._client_id = None
        self._workspace_id = None
        self._element_id = None
        self._part_id = None
        self._project_id = None
        self._folder_id = None
        self._version_id = None
        self._document_id = None
        self._url = None
        self._events = None
        self.discriminator = None

        if id is not None:
            self.id = id
        if options is not None:
            self.options = options
        if filter is not None:
            self.filter = filter
        if user_id is not None:
            self.user_id = user_id
        if data is not None:
            self.data = data
        if company_id is not None:
            self.company_id = company_id
        if client_id is not None:
            self.client_id = client_id
        if workspace_id is not None:
            self.workspace_id = workspace_id
        if element_id is not None:
            self.element_id = element_id
        if part_id is not None:
            self.part_id = part_id
        if project_id is not None:
            self.project_id = project_id
        if folder_id is not None:
            self.folder_id = folder_id
        if version_id is not None:
            self.version_id = version_id
        if document_id is not None:
            self.document_id = document_id
        if url is not None:
            self.url = url
        if events is not None:
            self.events = events

    @property
    def id(self):
        """Gets the id of this BTWebhookParams.  # noqa: E501


        :return: The id of this BTWebhookParams.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this BTWebhookParams.


        :param id: The id of this BTWebhookParams.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def options(self):
        """Gets the options of this BTWebhookParams.  # noqa: E501


        :return: The options of this BTWebhookParams.  # noqa: E501
        :rtype: BTWebhookOptions
        """
        return self._options

    @options.setter
    def options(self, options):
        """Sets the options of this BTWebhookParams.


        :param options: The options of this BTWebhookParams.  # noqa: E501
        :type: BTWebhookOptions
        """

        self._options = options

    @property
    def filter(self):
        """Gets the filter of this BTWebhookParams.  # noqa: E501


        :return: The filter of this BTWebhookParams.  # noqa: E501
        :rtype: str
        """
        return self._filter

    @filter.setter
    def filter(self, filter):
        """Sets the filter of this BTWebhookParams.


        :param filter: The filter of this BTWebhookParams.  # noqa: E501
        :type: str
        """

        self._filter = filter

    @property
    def user_id(self):
        """Gets the user_id of this BTWebhookParams.  # noqa: E501


        :return: The user_id of this BTWebhookParams.  # noqa: E501
        :rtype: str
        """
        return self._user_id

    @user_id.setter
    def user_id(self, user_id):
        """Sets the user_id of this BTWebhookParams.


        :param user_id: The user_id of this BTWebhookParams.  # noqa: E501
        :type: str
        """

        self._user_id = user_id

    @property
    def data(self):
        """Gets the data of this BTWebhookParams.  # noqa: E501


        :return: The data of this BTWebhookParams.  # noqa: E501
        :rtype: str
        """
        return self._data

    @data.setter
    def data(self, data):
        """Sets the data of this BTWebhookParams.


        :param data: The data of this BTWebhookParams.  # noqa: E501
        :type: str
        """

        self._data = data

    @property
    def company_id(self):
        """Gets the company_id of this BTWebhookParams.  # noqa: E501


        :return: The company_id of this BTWebhookParams.  # noqa: E501
        :rtype: str
        """
        return self._company_id

    @company_id.setter
    def company_id(self, company_id):
        """Sets the company_id of this BTWebhookParams.


        :param company_id: The company_id of this BTWebhookParams.  # noqa: E501
        :type: str
        """

        self._company_id = company_id

    @property
    def client_id(self):
        """Gets the client_id of this BTWebhookParams.  # noqa: E501


        :return: The client_id of this BTWebhookParams.  # noqa: E501
        :rtype: str
        """
        return self._client_id

    @client_id.setter
    def client_id(self, client_id):
        """Sets the client_id of this BTWebhookParams.


        :param client_id: The client_id of this BTWebhookParams.  # noqa: E501
        :type: str
        """

        self._client_id = client_id

    @property
    def workspace_id(self):
        """Gets the workspace_id of this BTWebhookParams.  # noqa: E501


        :return: The workspace_id of this BTWebhookParams.  # noqa: E501
        :rtype: str
        """
        return self._workspace_id

    @workspace_id.setter
    def workspace_id(self, workspace_id):
        """Sets the workspace_id of this BTWebhookParams.


        :param workspace_id: The workspace_id of this BTWebhookParams.  # noqa: E501
        :type: str
        """

        self._workspace_id = workspace_id

    @property
    def element_id(self):
        """Gets the element_id of this BTWebhookParams.  # noqa: E501


        :return: The element_id of this BTWebhookParams.  # noqa: E501
        :rtype: str
        """
        return self._element_id

    @element_id.setter
    def element_id(self, element_id):
        """Sets the element_id of this BTWebhookParams.


        :param element_id: The element_id of this BTWebhookParams.  # noqa: E501
        :type: str
        """

        self._element_id = element_id

    @property
    def part_id(self):
        """Gets the part_id of this BTWebhookParams.  # noqa: E501


        :return: The part_id of this BTWebhookParams.  # noqa: E501
        :rtype: str
        """
        return self._part_id

    @part_id.setter
    def part_id(self, part_id):
        """Sets the part_id of this BTWebhookParams.


        :param part_id: The part_id of this BTWebhookParams.  # noqa: E501
        :type: str
        """

        self._part_id = part_id

    @property
    def project_id(self):
        """Gets the project_id of this BTWebhookParams.  # noqa: E501


        :return: The project_id of this BTWebhookParams.  # noqa: E501
        :rtype: str
        """
        return self._project_id

    @project_id.setter
    def project_id(self, project_id):
        """Sets the project_id of this BTWebhookParams.


        :param project_id: The project_id of this BTWebhookParams.  # noqa: E501
        :type: str
        """

        self._project_id = project_id

    @property
    def folder_id(self):
        """Gets the folder_id of this BTWebhookParams.  # noqa: E501


        :return: The folder_id of this BTWebhookParams.  # noqa: E501
        :rtype: str
        """
        return self._folder_id

    @folder_id.setter
    def folder_id(self, folder_id):
        """Sets the folder_id of this BTWebhookParams.


        :param folder_id: The folder_id of this BTWebhookParams.  # noqa: E501
        :type: str
        """

        self._folder_id = folder_id

    @property
    def version_id(self):
        """Gets the version_id of this BTWebhookParams.  # noqa: E501


        :return: The version_id of this BTWebhookParams.  # noqa: E501
        :rtype: str
        """
        return self._version_id

    @version_id.setter
    def version_id(self, version_id):
        """Sets the version_id of this BTWebhookParams.


        :param version_id: The version_id of this BTWebhookParams.  # noqa: E501
        :type: str
        """

        self._version_id = version_id

    @property
    def document_id(self):
        """Gets the document_id of this BTWebhookParams.  # noqa: E501


        :return: The document_id of this BTWebhookParams.  # noqa: E501
        :rtype: str
        """
        return self._document_id

    @document_id.setter
    def document_id(self, document_id):
        """Sets the document_id of this BTWebhookParams.


        :param document_id: The document_id of this BTWebhookParams.  # noqa: E501
        :type: str
        """

        self._document_id = document_id

    @property
    def url(self):
        """Gets the url of this BTWebhookParams.  # noqa: E501


        :return: The url of this BTWebhookParams.  # noqa: E501
        :rtype: str
        """
        return self._url

    @url.setter
    def url(self, url):
        """Sets the url of this BTWebhookParams.


        :param url: The url of this BTWebhookParams.  # noqa: E501
        :type: str
        """

        self._url = url

    @property
    def events(self):
        """Gets the events of this BTWebhookParams.  # noqa: E501


        :return: The events of this BTWebhookParams.  # noqa: E501
        :rtype: list[str]
        """
        return self._events

    @events.setter
    def events(self, events):
        """Sets the events of this BTWebhookParams.


        :param events: The events of this BTWebhookParams.  # noqa: E501
        :type: list[str]
        """

        self._events = events

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, BTWebhookParams):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Onshape REST API

    The Onshape REST API consumed by all clients.  # noqa: E501

    OpenAPI spec version: 1.96
    Contact: api-support@onshape.zendesk.com
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class BTAutoPartNumberingSchemeParams(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'scheme_id': 'str',
        'assembly_prefix': 'str',
        'drawing_prefix': 'str',
        'file_prefix': 'str',
        'part_prefix': 'str',
        'next_number': 'int',
        'next_number_min_width': 'int'
    }

    attribute_map = {
        'scheme_id': 'schemeId',
        'assembly_prefix': 'assemblyPrefix',
        'drawing_prefix': 'drawingPrefix',
        'file_prefix': 'filePrefix',
        'part_prefix': 'partPrefix',
        'next_number': 'nextNumber',
        'next_number_min_width': 'nextNumberMinWidth'
    }

    def __init__(self, scheme_id=None, assembly_prefix=None, drawing_prefix=None, file_prefix=None, part_prefix=None, next_number=None, next_number_min_width=None):  # noqa: E501
        """BTAutoPartNumberingSchemeParams - a model defined in OpenAPI"""  # noqa: E501

        self._scheme_id = None
        self._assembly_prefix = None
        self._drawing_prefix = None
        self._file_prefix = None
        self._part_prefix = None
        self._next_number = None
        self._next_number_min_width = None
        self.discriminator = None

        if scheme_id is not None:
            self.scheme_id = scheme_id
        if assembly_prefix is not None:
            self.assembly_prefix = assembly_prefix
        if drawing_prefix is not None:
            self.drawing_prefix = drawing_prefix
        if file_prefix is not None:
            self.file_prefix = file_prefix
        if part_prefix is not None:
            self.part_prefix = part_prefix
        if next_number is not None:
            self.next_number = next_number
        if next_number_min_width is not None:
            self.next_number_min_width = next_number_min_width

    @property
    def scheme_id(self):
        """Gets the scheme_id of this BTAutoPartNumberingSchemeParams.  # noqa: E501


        :return: The scheme_id of this BTAutoPartNumberingSchemeParams.  # noqa: E501
        :rtype: str
        """
        return self._scheme_id

    @scheme_id.setter
    def scheme_id(self, scheme_id):
        """Sets the scheme_id of this BTAutoPartNumberingSchemeParams.


        :param scheme_id: The scheme_id of this BTAutoPartNumberingSchemeParams.  # noqa: E501
        :type: str
        """

        self._scheme_id = scheme_id

    @property
    def assembly_prefix(self):
        """Gets the assembly_prefix of this BTAutoPartNumberingSchemeParams.  # noqa: E501


        :return: The assembly_prefix of this BTAutoPartNumberingSchemeParams.  # noqa: E501
        :rtype: str
        """
        return self._assembly_prefix

    @assembly_prefix.setter
    def assembly_prefix(self, assembly_prefix):
        """Sets the assembly_prefix of this BTAutoPartNumberingSchemeParams.


        :param assembly_prefix: The assembly_prefix of this BTAutoPartNumberingSchemeParams.  # noqa: E501
        :type: str
        """

        self._assembly_prefix = assembly_prefix

    @property
    def drawing_prefix(self):
        """Gets the drawing_prefix of this BTAutoPartNumberingSchemeParams.  # noqa: E501


        :return: The drawing_prefix of this BTAutoPartNumberingSchemeParams.  # noqa: E501
        :rtype: str
        """
        return self._drawing_prefix

    @drawing_prefix.setter
    def drawing_prefix(self, drawing_prefix):
        """Sets the drawing_prefix of this BTAutoPartNumberingSchemeParams.


        :param drawing_prefix: The drawing_prefix of this BTAutoPartNumberingSchemeParams.  # noqa: E501
        :type: str
        """

        self._drawing_prefix = drawing_prefix

    @property
    def file_prefix(self):
        """Gets the file_prefix of this BTAutoPartNumberingSchemeParams.  # noqa: E501


        :return: The file_prefix of this BTAutoPartNumberingSchemeParams.  # noqa: E501
        :rtype: str
        """
        return self._file_prefix

    @file_prefix.setter
    def file_prefix(self, file_prefix):
        """Sets the file_prefix of this BTAutoPartNumberingSchemeParams.


        :param file_prefix: The file_prefix of this BTAutoPartNumberingSchemeParams.  # noqa: E501
        :type: str
        """

        self._file_prefix = file_prefix

    @property
    def part_prefix(self):
        """Gets the part_prefix of this BTAutoPartNumberingSchemeParams.  # noqa: E501


        :return: The part_prefix of this BTAutoPartNumberingSchemeParams.  # noqa: E501
        :rtype: str
        """
        return self._part_prefix

    @part_prefix.setter
    def part_prefix(self, part_prefix):
        """Sets the part_prefix of this BTAutoPartNumberingSchemeParams.


        :param part_prefix: The part_prefix of this BTAutoPartNumberingSchemeParams.  # noqa: E501
        :type: str
        """

        self._part_prefix = part_prefix

    @property
    def next_number(self):
        """Gets the next_number of this BTAutoPartNumberingSchemeParams.  # noqa: E501


        :return: The next_number of this BTAutoPartNumberingSchemeParams.  # noqa: E501
        :rtype: int
        """
        return self._next_number

    @next_number.setter
    def next_number(self, next_number):
        """Sets the next_number of this BTAutoPartNumberingSchemeParams.


        :param next_number: The next_number of this BTAutoPartNumberingSchemeParams.  # noqa: E501
        :type: int
        """

        self._next_number = next_number

    @property
    def next_number_min_width(self):
        """Gets the next_number_min_width of this BTAutoPartNumberingSchemeParams.  # noqa: E501


        :return: The next_number_min_width of this BTAutoPartNumberingSchemeParams.  # noqa: E501
        :rtype: int
        """
        return self._next_number_min_width

    @next_number_min_width.setter
    def next_number_min_width(self, next_number_min_width):
        """Sets the next_number_min_width of this BTAutoPartNumberingSchemeParams.


        :param next_number_min_width: The next_number_min_width of this BTAutoPartNumberingSchemeParams.  # noqa: E501
        :type: int
        """

        self._next_number_min_width = next_number_min_width

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, BTAutoPartNumberingSchemeParams):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Onshape REST API

    The Onshape REST API consumed by all clients.  # noqa: E501

    OpenAPI spec version: 1.96
    Contact: api-support@onshape.zendesk.com
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class BTApplicationElementThumbnailParams(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'description': 'str',
        'is_primary': 'bool',
        'unique_id': 'str',
        'base64_encoded_image': 'str',
        'image_width': 'int',
        'image_height': 'int'
    }

    attribute_map = {
        'description': 'description',
        'is_primary': 'isPrimary',
        'unique_id': 'uniqueId',
        'base64_encoded_image': 'base64EncodedImage',
        'image_width': 'imageWidth',
        'image_height': 'imageHeight'
    }

    def __init__(self, description=None, is_primary=None, unique_id=None, base64_encoded_image=None, image_width=None, image_height=None):  # noqa: E501
        """BTApplicationElementThumbnailParams - a model defined in OpenAPI"""  # noqa: E501

        self._description = None
        self._is_primary = None
        self._unique_id = None
        self._base64_encoded_image = None
        self._image_width = None
        self._image_height = None
        self.discriminator = None

        if description is not None:
            self.description = description
        if is_primary is not None:
            self.is_primary = is_primary
        if unique_id is not None:
            self.unique_id = unique_id
        if base64_encoded_image is not None:
            self.base64_encoded_image = base64_encoded_image
        if image_width is not None:
            self.image_width = image_width
        if image_height is not None:
            self.image_height = image_height

    @property
    def description(self):
        """Gets the description of this BTApplicationElementThumbnailParams.  # noqa: E501


        :return: The description of this BTApplicationElementThumbnailParams.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this BTApplicationElementThumbnailParams.


        :param description: The description of this BTApplicationElementThumbnailParams.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def is_primary(self):
        """Gets the is_primary of this BTApplicationElementThumbnailParams.  # noqa: E501


        :return: The is_primary of this BTApplicationElementThumbnailParams.  # noqa: E501
        :rtype: bool
        """
        return self._is_primary

    @is_primary.setter
    def is_primary(self, is_primary):
        """Sets the is_primary of this BTApplicationElementThumbnailParams.


        :param is_primary: The is_primary of this BTApplicationElementThumbnailParams.  # noqa: E501
        :type: bool
        """

        self._is_primary = is_primary

    @property
    def unique_id(self):
        """Gets the unique_id of this BTApplicationElementThumbnailParams.  # noqa: E501


        :return: The unique_id of this BTApplicationElementThumbnailParams.  # noqa: E501
        :rtype: str
        """
        return self._unique_id

    @unique_id.setter
    def unique_id(self, unique_id):
        """Sets the unique_id of this BTApplicationElementThumbnailParams.


        :param unique_id: The unique_id of this BTApplicationElementThumbnailParams.  # noqa: E501
        :type: str
        """

        self._unique_id = unique_id

    @property
    def base64_encoded_image(self):
        """Gets the base64_encoded_image of this BTApplicationElementThumbnailParams.  # noqa: E501


        :return: The base64_encoded_image of this BTApplicationElementThumbnailParams.  # noqa: E501
        :rtype: str
        """
        return self._base64_encoded_image

    @base64_encoded_image.setter
    def base64_encoded_image(self, base64_encoded_image):
        """Sets the base64_encoded_image of this BTApplicationElementThumbnailParams.


        :param base64_encoded_image: The base64_encoded_image of this BTApplicationElementThumbnailParams.  # noqa: E501
        :type: str
        """

        self._base64_encoded_image = base64_encoded_image

    @property
    def image_width(self):
        """Gets the image_width of this BTApplicationElementThumbnailParams.  # noqa: E501


        :return: The image_width of this BTApplicationElementThumbnailParams.  # noqa: E501
        :rtype: int
        """
        return self._image_width

    @image_width.setter
    def image_width(self, image_width):
        """Sets the image_width of this BTApplicationElementThumbnailParams.


        :param image_width: The image_width of this BTApplicationElementThumbnailParams.  # noqa: E501
        :type: int
        """

        self._image_width = image_width

    @property
    def image_height(self):
        """Gets the image_height of this BTApplicationElementThumbnailParams.  # noqa: E501


        :return: The image_height of this BTApplicationElementThumbnailParams.  # noqa: E501
        :rtype: int
        """
        return self._image_height

    @image_height.setter
    def image_height(self, image_height):
        """Sets the image_height of this BTApplicationElementThumbnailParams.


        :param image_height: The image_height of this BTApplicationElementThumbnailParams.  # noqa: E501
        :type: int
        """

        self._image_height = image_height

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, BTApplicationElementThumbnailParams):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



"""This is for storing all the Onshape-specific functionality added so it is all in one place"""
import six
import re
from onshape_client.compatible_imports import unquote

def decode_file_name_from_header(content_disposition_header):
    filename_potential = re.search(r"""filename\*?=([^\s]+)""",
                         content_disposition_header)
    if "UTF-8" in filename_potential.group(1):
        filename_potential = re.search(r"""UTF-8[^\d\%]*([^\'\s]+)""", content_disposition_header)
        filename = unquote(filename_potential.group(1))
    else:
        filename = filename_potential.group(1).replace('"', '').replace("'", "")
    return filename


import pytest
from onshape_client.client import Client
from onshape_client.onshape_url import ConfiguredOnshapeElement

@pytest.fixture
def client():
    try:
        client = Client.get_client()
    except Exception as e:
        client = Client()
    return client

@pytest.fixture
def configurable_cube():
    return ConfiguredOnshapeElement("https://cad.onshape.com/documents/cca81d10f239db0db9481e6f/v/ca51b7554314d6aab254d2e6/e/69c9eedda86512966b20bc90")

@pytest.fixture
def configurable_cubes():
    versions=[]
    versions.append(ConfiguredOnshapeElement("https://cad.onshape.com/documents/cca81d10f239db0db9481e6f/v/2c9608e8e443574edf757851/e/69c9eedda86512966b20bc90"))
    versions.append(ConfiguredOnshapeElement("https://cad.onshape.com/documents/cca81d10f239db0db9481e6f/v/e21a30638b85226b651e74f9/e/69c9eedda86512966b20bc90"))
    versions.append(ConfiguredOnshapeElement("https://cad.onshape.com/documents/cca81d10f239db0db9481e6f/v/ca51b7554314d6aab254d2e6/e/69c9eedda86512966b20bc90"))
    versions.append(ConfiguredOnshapeElement("https://cad.onshape.com/documents/cca81d10f239db0db9481e6f/v/aaa25d18038e06f0b2964e2d/e/69c9eedda86512966b20bc90"))
    return versions


# coding: utf-8

"""
    Onshape REST API

    The Onshape REST API consumed by all clients.  # noqa: E501

    OpenAPI spec version: 1.96
    Contact: api-support@onshape.zendesk.com
    Generated by: https://openapi-generator.tech
"""


from setuptools import setup, find_packages  # noqa: H301

NAME = "onshape_client"
VERSION = "0.0.17"
# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools

REQUIRES = ["urllib3 >= 1.15", "six >= 1.10", "certifi", "python-dateutil", "requests_oauthlib", "ruamel.yaml", "pathlib", "enum34", "pint"]

setup(
    name=NAME,
    version=VERSION,
    description="Onshape REST API",
    author_email="api-support@onshape.zendesk.com",
    url="",
    keywords=["OpenAPI", "OpenAPI-Generator", "Onshape REST API"],
    install_requires=REQUIRES,
    packages=find_packages(),
    include_package_data=True,
    long_description="""\
    The Onshape REST API consumed by all clients.  # noqa: E501
    """
)



from onshape_client.client import Client

client = Client(stack_key="prod-oauth-full")

def test_client(configurable_cube):
    client.elements_api.get_configuration3(configurable_cube.did, configurable_cube.wvm, configurable_cube.wvmid,
                                           configurable_cube.eid)



# coding: utf-8

"""
    Onshape REST API

    The Onshape REST API consumed by all clients.  # noqa: E501

    OpenAPI spec version: 1.96
    Contact: api-support@onshape.zendesk.com
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class SubscriptionItemCollection(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'data': 'list[SubscriptionItem]',
        'total_count': 'int',
        'has_more': 'bool',
        'request_options': 'RequestOptions',
        'request_params': 'dict(str, object)',
        'url': 'str',
        'count': 'int'
    }

    attribute_map = {
        'data': 'data',
        'total_count': 'totalCount',
        'has_more': 'hasMore',
        'request_options': 'requestOptions',
        'request_params': 'requestParams',
        'url': 'url',
        'count': 'count'
    }

    def __init__(self, data=None, total_count=None, has_more=None, request_options=None, request_params=None, url=None, count=None):  # noqa: E501
        """SubscriptionItemCollection - a model defined in OpenAPI"""  # noqa: E501

        self._data = None
        self._total_count = None
        self._has_more = None
        self._request_options = None
        self._request_params = None
        self._url = None
        self._count = None
        self.discriminator = None

        if data is not None:
            self.data = data
        if total_count is not None:
            self.total_count = total_count
        if has_more is not None:
            self.has_more = has_more
        if request_options is not None:
            self.request_options = request_options
        if request_params is not None:
            self.request_params = request_params
        if url is not None:
            self.url = url
        if count is not None:
            self.count = count

    @property
    def data(self):
        """Gets the data of this SubscriptionItemCollection.  # noqa: E501


        :return: The data of this SubscriptionItemCollection.  # noqa: E501
        :rtype: list[SubscriptionItem]
        """
        return self._data

    @data.setter
    def data(self, data):
        """Sets the data of this SubscriptionItemCollection.


        :param data: The data of this SubscriptionItemCollection.  # noqa: E501
        :type: list[SubscriptionItem]
        """

        self._data = data

    @property
    def total_count(self):
        """Gets the total_count of this SubscriptionItemCollection.  # noqa: E501


        :return: The total_count of this SubscriptionItemCollection.  # noqa: E501
        :rtype: int
        """
        return self._total_count

    @total_count.setter
    def total_count(self, total_count):
        """Sets the total_count of this SubscriptionItemCollection.


        :param total_count: The total_count of this SubscriptionItemCollection.  # noqa: E501
        :type: int
        """

        self._total_count = total_count

    @property
    def has_more(self):
        """Gets the has_more of this SubscriptionItemCollection.  # noqa: E501


        :return: The has_more of this SubscriptionItemCollection.  # noqa: E501
        :rtype: bool
        """
        return self._has_more

    @has_more.setter
    def has_more(self, has_more):
        """Sets the has_more of this SubscriptionItemCollection.


        :param has_more: The has_more of this SubscriptionItemCollection.  # noqa: E501
        :type: bool
        """

        self._has_more = has_more

    @property
    def request_options(self):
        """Gets the request_options of this SubscriptionItemCollection.  # noqa: E501


        :return: The request_options of this SubscriptionItemCollection.  # noqa: E501
        :rtype: RequestOptions
        """
        return self._request_options

    @request_options.setter
    def request_options(self, request_options):
        """Sets the request_options of this SubscriptionItemCollection.


        :param request_options: The request_options of this SubscriptionItemCollection.  # noqa: E501
        :type: RequestOptions
        """

        self._request_options = request_options

    @property
    def request_params(self):
        """Gets the request_params of this SubscriptionItemCollection.  # noqa: E501


        :return: The request_params of this SubscriptionItemCollection.  # noqa: E501
        :rtype: dict(str, object)
        """
        return self._request_params

    @request_params.setter
    def request_params(self, request_params):
        """Sets the request_params of this SubscriptionItemCollection.


        :param request_params: The request_params of this SubscriptionItemCollection.  # noqa: E501
        :type: dict(str, object)
        """

        self._request_params = request_params

    @property
    def url(self):
        """Gets the url of this SubscriptionItemCollection.  # noqa: E501


        :return: The url of this SubscriptionItemCollection.  # noqa: E501
        :rtype: str
        """
        return self._url

    @url.setter
    def url(self, url):
        """Sets the url of this SubscriptionItemCollection.


        :param url: The url of this SubscriptionItemCollection.  # noqa: E501
        :type: str
        """

        self._url = url

    @property
    def count(self):
        """Gets the count of this SubscriptionItemCollection.  # noqa: E501


        :return: The count of this SubscriptionItemCollection.  # noqa: E501
        :rtype: int
        """
        return self._count

    @count.setter
    def count(self, count):
        """Sets the count of this SubscriptionItemCollection.


        :param count: The count of this SubscriptionItemCollection.  # noqa: E501
        :type: int
        """

        self._count = count

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SubscriptionItemCollection):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Onshape REST API

    The Onshape REST API consumed by all clients.  # noqa: E501

    OpenAPI spec version: 1.96
    Contact: api-support@onshape.zendesk.com
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class BTCommentParams(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'message': 'str',
        'id': 'str',
        'view_data': 'BTViewDataParams',
        'element_query': 'str',
        'element_feature': 'str',
        'element_occurrence': 'str',
        'assembly_feature': 'str',
        'workspace_id': 'str',
        'element_id': 'str',
        'parent_id': 'str',
        'version_id': 'str',
        'document_id': 'str',
        'assignee': 'str'
    }

    attribute_map = {
        'message': 'message',
        'id': 'id',
        'view_data': 'viewData',
        'element_query': 'elementQuery',
        'element_feature': 'elementFeature',
        'element_occurrence': 'elementOccurrence',
        'assembly_feature': 'assemblyFeature',
        'workspace_id': 'workspaceId',
        'element_id': 'elementId',
        'parent_id': 'parentId',
        'version_id': 'versionId',
        'document_id': 'documentId',
        'assignee': 'assignee'
    }

    def __init__(self, message=None, id=None, view_data=None, element_query=None, element_feature=None, element_occurrence=None, assembly_feature=None, workspace_id=None, element_id=None, parent_id=None, version_id=None, document_id=None, assignee=None):  # noqa: E501
        """BTCommentParams - a model defined in OpenAPI"""  # noqa: E501

        self._message = None
        self._id = None
        self._view_data = None
        self._element_query = None
        self._element_feature = None
        self._element_occurrence = None
        self._assembly_feature = None
        self._workspace_id = None
        self._element_id = None
        self._parent_id = None
        self._version_id = None
        self._document_id = None
        self._assignee = None
        self.discriminator = None

        if message is not None:
            self.message = message
        if id is not None:
            self.id = id
        if view_data is not None:
            self.view_data = view_data
        if element_query is not None:
            self.element_query = element_query
        if element_feature is not None:
            self.element_feature = element_feature
        if element_occurrence is not None:
            self.element_occurrence = element_occurrence
        if assembly_feature is not None:
            self.assembly_feature = assembly_feature
        if workspace_id is not None:
            self.workspace_id = workspace_id
        if element_id is not None:
            self.element_id = element_id
        if parent_id is not None:
            self.parent_id = parent_id
        if version_id is not None:
            self.version_id = version_id
        if document_id is not None:
            self.document_id = document_id
        if assignee is not None:
            self.assignee = assignee

    @property
    def message(self):
        """Gets the message of this BTCommentParams.  # noqa: E501


        :return: The message of this BTCommentParams.  # noqa: E501
        :rtype: str
        """
        return self._message

    @message.setter
    def message(self, message):
        """Sets the message of this BTCommentParams.


        :param message: The message of this BTCommentParams.  # noqa: E501
        :type: str
        """

        self._message = message

    @property
    def id(self):
        """Gets the id of this BTCommentParams.  # noqa: E501


        :return: The id of this BTCommentParams.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this BTCommentParams.


        :param id: The id of this BTCommentParams.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def view_data(self):
        """Gets the view_data of this BTCommentParams.  # noqa: E501


        :return: The view_data of this BTCommentParams.  # noqa: E501
        :rtype: BTViewDataParams
        """
        return self._view_data

    @view_data.setter
    def view_data(self, view_data):
        """Sets the view_data of this BTCommentParams.


        :param view_data: The view_data of this BTCommentParams.  # noqa: E501
        :type: BTViewDataParams
        """

        self._view_data = view_data

    @property
    def element_query(self):
        """Gets the element_query of this BTCommentParams.  # noqa: E501


        :return: The element_query of this BTCommentParams.  # noqa: E501
        :rtype: str
        """
        return self._element_query

    @element_query.setter
    def element_query(self, element_query):
        """Sets the element_query of this BTCommentParams.


        :param element_query: The element_query of this BTCommentParams.  # noqa: E501
        :type: str
        """

        self._element_query = element_query

    @property
    def element_feature(self):
        """Gets the element_feature of this BTCommentParams.  # noqa: E501


        :return: The element_feature of this BTCommentParams.  # noqa: E501
        :rtype: str
        """
        return self._element_feature

    @element_feature.setter
    def element_feature(self, element_feature):
        """Sets the element_feature of this BTCommentParams.


        :param element_feature: The element_feature of this BTCommentParams.  # noqa: E501
        :type: str
        """

        self._element_feature = element_feature

    @property
    def element_occurrence(self):
        """Gets the element_occurrence of this BTCommentParams.  # noqa: E501


        :return: The element_occurrence of this BTCommentParams.  # noqa: E501
        :rtype: str
        """
        return self._element_occurrence

    @element_occurrence.setter
    def element_occurrence(self, element_occurrence):
        """Sets the element_occurrence of this BTCommentParams.


        :param element_occurrence: The element_occurrence of this BTCommentParams.  # noqa: E501
        :type: str
        """

        self._element_occurrence = element_occurrence

    @property
    def assembly_feature(self):
        """Gets the assembly_feature of this BTCommentParams.  # noqa: E501


        :return: The assembly_feature of this BTCommentParams.  # noqa: E501
        :rtype: str
        """
        return self._assembly_feature

    @assembly_feature.setter
    def assembly_feature(self, assembly_feature):
        """Sets the assembly_feature of this BTCommentParams.


        :param assembly_feature: The assembly_feature of this BTCommentParams.  # noqa: E501
        :type: str
        """

        self._assembly_feature = assembly_feature

    @property
    def workspace_id(self):
        """Gets the workspace_id of this BTCommentParams.  # noqa: E501


        :return: The workspace_id of this BTCommentParams.  # noqa: E501
        :rtype: str
        """
        return self._workspace_id

    @workspace_id.setter
    def workspace_id(self, workspace_id):
        """Sets the workspace_id of this BTCommentParams.


        :param workspace_id: The workspace_id of this BTCommentParams.  # noqa: E501
        :type: str
        """

        self._workspace_id = workspace_id

    @property
    def element_id(self):
        """Gets the element_id of this BTCommentParams.  # noqa: E501


        :return: The element_id of this BTCommentParams.  # noqa: E501
        :rtype: str
        """
        return self._element_id

    @element_id.setter
    def element_id(self, element_id):
        """Sets the element_id of this BTCommentParams.


        :param element_id: The element_id of this BTCommentParams.  # noqa: E501
        :type: str
        """

        self._element_id = element_id

    @property
    def parent_id(self):
        """Gets the parent_id of this BTCommentParams.  # noqa: E501


        :return: The parent_id of this BTCommentParams.  # noqa: E501
        :rtype: str
        """
        return self._parent_id

    @parent_id.setter
    def parent_id(self, parent_id):
        """Sets the parent_id of this BTCommentParams.


        :param parent_id: The parent_id of this BTCommentParams.  # noqa: E501
        :type: str
        """

        self._parent_id = parent_id

    @property
    def version_id(self):
        """Gets the version_id of this BTCommentParams.  # noqa: E501


        :return: The version_id of this BTCommentParams.  # noqa: E501
        :rtype: str
        """
        return self._version_id

    @version_id.setter
    def version_id(self, version_id):
        """Sets the version_id of this BTCommentParams.


        :param version_id: The version_id of this BTCommentParams.  # noqa: E501
        :type: str
        """

        self._version_id = version_id

    @property
    def document_id(self):
        """Gets the document_id of this BTCommentParams.  # noqa: E501


        :return: The document_id of this BTCommentParams.  # noqa: E501
        :rtype: str
        """
        return self._document_id

    @document_id.setter
    def document_id(self, document_id):
        """Sets the document_id of this BTCommentParams.


        :param document_id: The document_id of this BTCommentParams.  # noqa: E501
        :type: str
        """

        self._document_id = document_id

    @property
    def assignee(self):
        """Gets the assignee of this BTCommentParams.  # noqa: E501


        :return: The assignee of this BTCommentParams.  # noqa: E501
        :rtype: str
        """
        return self._assignee

    @assignee.setter
    def assignee(self, assignee):
        """Sets the assignee of this BTCommentParams.


        :param assignee: The assignee of this BTCommentParams.  # noqa: E501
        :type: str
        """

        self._assignee = assignee

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, BTCommentParams):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Onshape REST API

    The Onshape REST API consumed by all clients.  # noqa: E501

    OpenAPI spec version: 1.96
    Contact: api-support@onshape.zendesk.com
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class BTUserSettingsParams(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'value': 'str',
        'key': 'str',
        'id': 'str',
        'key_values': 'dict(str, str)',
        'substitute_approvers': 'list[BTSubstituteApproverParams]'
    }

    attribute_map = {
        'value': 'value',
        'key': 'key',
        'id': 'id',
        'key_values': 'keyValues',
        'substitute_approvers': 'substituteApprovers'
    }

    def __init__(self, value=None, key=None, id=None, key_values=None, substitute_approvers=None):  # noqa: E501
        """BTUserSettingsParams - a model defined in OpenAPI"""  # noqa: E501

        self._value = None
        self._key = None
        self._id = None
        self._key_values = None
        self._substitute_approvers = None
        self.discriminator = None

        if value is not None:
            self.value = value
        if key is not None:
            self.key = key
        if id is not None:
            self.id = id
        if key_values is not None:
            self.key_values = key_values
        if substitute_approvers is not None:
            self.substitute_approvers = substitute_approvers

    @property
    def value(self):
        """Gets the value of this BTUserSettingsParams.  # noqa: E501


        :return: The value of this BTUserSettingsParams.  # noqa: E501
        :rtype: str
        """
        return self._value

    @value.setter
    def value(self, value):
        """Sets the value of this BTUserSettingsParams.


        :param value: The value of this BTUserSettingsParams.  # noqa: E501
        :type: str
        """

        self._value = value

    @property
    def key(self):
        """Gets the key of this BTUserSettingsParams.  # noqa: E501


        :return: The key of this BTUserSettingsParams.  # noqa: E501
        :rtype: str
        """
        return self._key

    @key.setter
    def key(self, key):
        """Sets the key of this BTUserSettingsParams.


        :param key: The key of this BTUserSettingsParams.  # noqa: E501
        :type: str
        """

        self._key = key

    @property
    def id(self):
        """Gets the id of this BTUserSettingsParams.  # noqa: E501


        :return: The id of this BTUserSettingsParams.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this BTUserSettingsParams.


        :param id: The id of this BTUserSettingsParams.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def key_values(self):
        """Gets the key_values of this BTUserSettingsParams.  # noqa: E501


        :return: The key_values of this BTUserSettingsParams.  # noqa: E501
        :rtype: dict(str, str)
        """
        return self._key_values

    @key_values.setter
    def key_values(self, key_values):
        """Sets the key_values of this BTUserSettingsParams.


        :param key_values: The key_values of this BTUserSettingsParams.  # noqa: E501
        :type: dict(str, str)
        """

        self._key_values = key_values

    @property
    def substitute_approvers(self):
        """Gets the substitute_approvers of this BTUserSettingsParams.  # noqa: E501


        :return: The substitute_approvers of this BTUserSettingsParams.  # noqa: E501
        :rtype: list[BTSubstituteApproverParams]
        """
        return self._substitute_approvers

    @substitute_approvers.setter
    def substitute_approvers(self, substitute_approvers):
        """Sets the substitute_approvers of this BTUserSettingsParams.


        :param substitute_approvers: The substitute_approvers of this BTUserSettingsParams.  # noqa: E501
        :type: list[BTSubstituteApproverParams]
        """

        self._substitute_approvers = substitute_approvers

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, BTUserSettingsParams):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Onshape REST API

    The Onshape REST API consumed by all clients.  # noqa: E501

    OpenAPI spec version: 1.96
    Contact: api-support@onshape.zendesk.com
    Generated by: https://openapi-generator.tech
"""


from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from onshape_client.api_client import ApiClient


class DocumentsApi(object):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def download_external_data(self, did, fid, **kwargs):  # noqa: E501
        """Download External Data  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.download_external_data(did, fid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str did: (required)
        :param str fid: (required)
        :param str if_none_match:
        :return: file
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.download_external_data_with_http_info(did, fid, **kwargs)  # noqa: E501
        else:
            (data) = self.download_external_data_with_http_info(did, fid, **kwargs)  # noqa: E501
            return data

    def download_external_data_with_http_info(self, did, fid, **kwargs):  # noqa: E501
        """Download External Data  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.download_external_data_with_http_info(did, fid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str did: (required)
        :param str fid: (required)
        :param str if_none_match:
        :return: file
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = ['did', 'fid', 'if_none_match']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method download_external_data" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']
        # verify the required parameter 'did' is set
        if ('did' not in local_var_params or
                local_var_params['did'] is None):
            raise ValueError("Missing the required parameter `did` when calling `download_external_data`")  # noqa: E501
        # verify the required parameter 'fid' is set
        if ('fid' not in local_var_params or
                local_var_params['fid'] is None):
            raise ValueError("Missing the required parameter `fid` when calling `download_external_data`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'did' in local_var_params:
            path_params['did'] = local_var_params['did']  # noqa: E501
        if 'fid' in local_var_params:
            path_params['fid'] = local_var_params['fid']  # noqa: E501

        query_params = []

        header_params = {}
        if 'if_none_match' in local_var_params:
            header_params['If-None-Match'] = local_var_params['if_none_match']  # noqa: E501

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/vnd.onshape.v1+json;charset=UTF-8; qs=0.1', 'application/json;charset=UTF-8; qs=0.9', 'application/vnd.onshape.v1+octet-stream; qs=0.1', 'application/octet-stream'])  # noqa: E501

        # Authentication setting
        auth_settings = ['OAuth2']  # noqa: E501

        return self.api_client.call_api(
            '/api/documents/d/{did}/externaldata/{fid}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='file',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_current_microversion(self, did, wv, wvid, **kwargs):  # noqa: E501
        """get_current_microversion  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_current_microversion(did, wv, wvid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str did: (required)
        :param str wv: (required)
        :param str wvid: (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_current_microversion_with_http_info(did, wv, wvid, **kwargs)  # noqa: E501
        else:
            (data) = self.get_current_microversion_with_http_info(did, wv, wvid, **kwargs)  # noqa: E501
            return data

    def get_current_microversion_with_http_info(self, did, wv, wvid, **kwargs):  # noqa: E501
        """get_current_microversion  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_current_microversion_with_http_info(did, wv, wvid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str did: (required)
        :param str wv: (required)
        :param str wvid: (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = ['did', 'wv', 'wvid']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_current_microversion" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']
        # verify the required parameter 'did' is set
        if ('did' not in local_var_params or
                local_var_params['did'] is None):
            raise ValueError("Missing the required parameter `did` when calling `get_current_microversion`")  # noqa: E501
        # verify the required parameter 'wv' is set
        if ('wv' not in local_var_params or
                local_var_params['wv'] is None):
            raise ValueError("Missing the required parameter `wv` when calling `get_current_microversion`")  # noqa: E501
        # verify the required parameter 'wvid' is set
        if ('wvid' not in local_var_params or
                local_var_params['wvid'] is None):
            raise ValueError("Missing the required parameter `wvid` when calling `get_current_microversion`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'did' in local_var_params:
            path_params['did'] = local_var_params['did']  # noqa: E501
        if 'wv' in local_var_params:
            path_params['wv'] = local_var_params['wv']  # noqa: E501
        if 'wvid' in local_var_params:
            path_params['wvid'] = local_var_params['wvid']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/vnd.onshape.v1+json;charset=UTF-8; qs=0.1', 'application/json;charset=UTF-8; qs=0.9'])  # noqa: E501

        # Authentication setting
        auth_settings = ['OAuth2']  # noqa: E501

        return self.api_client.call_api(
            '/api/documents/d/{did}/{wv}/{wvid}/currentmicroversion', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type=None,  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_documents(self, **kwargs):  # noqa: E501
        """Get Documents  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_documents(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str q:
        :param int filter:
        :param str owner:
        :param int owner_type:
        :param str sort_column:
        :param str sort_order:
        :param int offset:
        :param int limit:
        :param str label:
        :param str project:
        :param str parent_id:
        :return: BTGlobalTreeNodeListResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_documents_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.get_documents_with_http_info(**kwargs)  # noqa: E501
            return data

    def get_documents_with_http_info(self, **kwargs):  # noqa: E501
        """Get Documents  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_documents_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str q:
        :param int filter:
        :param str owner:
        :param int owner_type:
        :param str sort_column:
        :param str sort_order:
        :param int offset:
        :param int limit:
        :param str label:
        :param str project:
        :param str parent_id:
        :return: BTGlobalTreeNodeListResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = ['q', 'filter', 'owner', 'owner_type', 'sort_column', 'sort_order', 'offset', 'limit', 'label', 'project', 'parent_id']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_documents" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'q' in local_var_params:
            query_params.append(('q', local_var_params['q']))  # noqa: E501
        if 'filter' in local_var_params:
            query_params.append(('filter', local_var_params['filter']))  # noqa: E501
        if 'owner' in local_var_params:
            query_params.append(('owner', local_var_params['owner']))  # noqa: E501
        if 'owner_type' in local_var_params:
            query_params.append(('ownerType', local_var_params['owner_type']))  # noqa: E501
        if 'sort_column' in local_var_params:
            query_params.append(('sortColumn', local_var_params['sort_column']))  # noqa: E501
        if 'sort_order' in local_var_params:
            query_params.append(('sortOrder', local_var_params['sort_order']))  # noqa: E501
        if 'offset' in local_var_params:
            query_params.append(('offset', local_var_params['offset']))  # noqa: E501
        if 'limit' in local_var_params:
            query_params.append(('limit', local_var_params['limit']))  # noqa: E501
        if 'label' in local_var_params:
            query_params.append(('label', local_var_params['label']))  # noqa: E501
        if 'project' in local_var_params:
            query_params.append(('project', local_var_params['project']))  # noqa: E501
        if 'parent_id' in local_var_params:
            query_params.append(('parentId', local_var_params['parent_id']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['OAuth2']  # noqa: E501

        return self.api_client.call_api(
            '/api/documents', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='BTGlobalTreeNodeListResponse',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats)

    def merge_into_workspace(self, did, wid, bt_version_or_workspace_info, **kwargs):  # noqa: E501
        """Merge into workspace  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.merge_into_workspace(did, wid, bt_version_or_workspace_info, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str did: (required)
        :param str wid: (required)
        :param BTVersionOrWorkspaceInfo bt_version_or_workspace_info: (required)
        :return: BTDocumentMergeInfo
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.merge_into_workspace_with_http_info(did, wid, bt_version_or_workspace_info, **kwargs)  # noqa: E501
        else:
            (data) = self.merge_into_workspace_with_http_info(did, wid, bt_version_or_workspace_info, **kwargs)  # noqa: E501
            return data

    def merge_into_workspace_with_http_info(self, did, wid, bt_version_or_workspace_info, **kwargs):  # noqa: E501
        """Merge into workspace  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.merge_into_workspace_with_http_info(did, wid, bt_version_or_workspace_info, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str did: (required)
        :param str wid: (required)
        :param BTVersionOrWorkspaceInfo bt_version_or_workspace_info: (required)
        :return: BTDocumentMergeInfo
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = ['did', 'wid', 'bt_version_or_workspace_info']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method merge_into_workspace" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']
        # verify the required parameter 'did' is set
        if ('did' not in local_var_params or
                local_var_params['did'] is None):
            raise ValueError("Missing the required parameter `did` when calling `merge_into_workspace`")  # noqa: E501
        # verify the required parameter 'wid' is set
        if ('wid' not in local_var_params or
                local_var_params['wid'] is None):
            raise ValueError("Missing the required parameter `wid` when calling `merge_into_workspace`")  # noqa: E501
        # verify the required parameter 'bt_version_or_workspace_info' is set
        if ('bt_version_or_workspace_info' not in local_var_params or
                local_var_params['bt_version_or_workspace_info'] is None):
            raise ValueError("Missing the required parameter `bt_version_or_workspace_info` when calling `merge_into_workspace`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'did' in local_var_params:
            path_params['did'] = local_var_params['did']  # noqa: E501
        if 'wid' in local_var_params:
            path_params['wid'] = local_var_params['wid']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        if 'bt_version_or_workspace_info' in local_var_params:
            body_params = local_var_params['bt_version_or_workspace_info']
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/vnd.onshape.v2+json;charset=UTF-8; qs=0.2'])  # noqa: E501

        # HTTP header `Content-Type`
        header_params['Content-Type'] = self.api_client.select_header_content_type(  # noqa: E501
            ['application/json;charset=UTF-8'])  # noqa: E501

        # Authentication setting
        auth_settings = ['OAuth2']  # noqa: E501

        return self.api_client.call_api(
            '/api/documents/{did}/workspaces/{wid}/merge', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='BTDocumentMergeInfo',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats)

    def restore_document(self, did, **kwargs):  # noqa: E501
        """Restore Document  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.restore_document(did, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str did: (required)
        :return: BTDocumentMergeInfo
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.restore_document_with_http_info(did, **kwargs)  # noqa: E501
        else:
            (data) = self.restore_document_with_http_info(did, **kwargs)  # noqa: E501
            return data

    def restore_document_with_http_info(self, did, **kwargs):  # noqa: E501
        """Restore Document  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.restore_document_with_http_info(did, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str did: (required)
        :return: BTDocumentMergeInfo
                 If the method is called asynchronously,
                 returns the request thread.
        """

        local_var_params = locals()

        all_params = ['did']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        for key, val in six.iteritems(local_var_params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method restore_document" % key
                )
            local_var_params[key] = val
        del local_var_params['kwargs']
        # verify the required parameter 'did' is set
        if ('did' not in local_var_params or
                local_var_params['did'] is None):
            raise ValueError("Missing the required parameter `did` when calling `restore_document`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'did' in local_var_params:
            path_params['did'] = local_var_params['did']  # noqa: E501

        query_params = []

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/vnd.onshape.v2+json;charset=UTF-8; qs=0.2'])  # noqa: E501

        # Authentication setting
        auth_settings = ['OAuth2']  # noqa: E501

        return self.api_client.call_api(
            '/api/documents/{did}/restore', 'POST',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='BTDocumentMergeInfo',  # noqa: E501
            auth_settings=auth_settings,
            async_req=local_var_params.get('async_req'),
            _return_http_data_only=local_var_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=local_var_params.get('_preload_content', True),
            _request_timeout=local_var_params.get('_request_timeout'),
            collection_formats=collection_formats)



# coding: utf-8

"""
    Onshape REST API

    The Onshape REST API consumed by all clients.  # noqa: E501

    OpenAPI spec version: 1.96
    Contact: api-support@onshape.zendesk.com
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class BTTeamMemberParams(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'email': 'str',
        'admin': 'bool',
        'team_id': 'str'
    }

    attribute_map = {
        'email': 'email',
        'admin': 'admin',
        'team_id': 'teamId'
    }

    def __init__(self, email=None, admin=None, team_id=None):  # noqa: E501
        """BTTeamMemberParams - a model defined in OpenAPI"""  # noqa: E501

        self._email = None
        self._admin = None
        self._team_id = None
        self.discriminator = None

        if email is not None:
            self.email = email
        if admin is not None:
            self.admin = admin
        if team_id is not None:
            self.team_id = team_id

    @property
    def email(self):
        """Gets the email of this BTTeamMemberParams.  # noqa: E501


        :return: The email of this BTTeamMemberParams.  # noqa: E501
        :rtype: str
        """
        return self._email

    @email.setter
    def email(self, email):
        """Sets the email of this BTTeamMemberParams.


        :param email: The email of this BTTeamMemberParams.  # noqa: E501
        :type: str
        """

        self._email = email

    @property
    def admin(self):
        """Gets the admin of this BTTeamMemberParams.  # noqa: E501


        :return: The admin of this BTTeamMemberParams.  # noqa: E501
        :rtype: bool
        """
        return self._admin

    @admin.setter
    def admin(self, admin):
        """Sets the admin of this BTTeamMemberParams.


        :param admin: The admin of this BTTeamMemberParams.  # noqa: E501
        :type: bool
        """

        self._admin = admin

    @property
    def team_id(self):
        """Gets the team_id of this BTTeamMemberParams.  # noqa: E501


        :return: The team_id of this BTTeamMemberParams.  # noqa: E501
        :rtype: str
        """
        return self._team_id

    @team_id.setter
    def team_id(self, team_id):
        """Sets the team_id of this BTTeamMemberParams.


        :param team_id: The team_id of this BTTeamMemberParams.  # noqa: E501
        :type: str
        """

        self._team_id = team_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, BTTeamMemberParams):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from __future__ import absolute_import

# flake8: noqa

# import apis into api package
from onshape_client.api.accounts_api import AccountsApi
from onshape_client.api.app_elements_api import AppElementsApi
from onshape_client.api.assemblies_api import AssembliesApi
from onshape_client.api.blob_elements_api import BlobElementsApi
from onshape_client.api.documents_api import DocumentsApi
from onshape_client.api.elements_api import ElementsApi
from onshape_client.api.endpoints_api import EndpointsApi
from onshape_client.api.metadata_api import MetadataApi
from onshape_client.api.part_studios_api import PartStudiosApi
from onshape_client.api.parts_api import PartsApi
from onshape_client.api.translations_api import TranslationsApi
from onshape_client.api.default_api import DefaultApi



# coding: utf-8

"""
    Onshape REST API

    The Onshape REST API consumed by all clients.  # noqa: E501

    OpenAPI spec version: 1.96
    Contact: api-support@onshape.zendesk.com
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class BTUserWebPreferencesParams(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'id': 'str',
        'user_id': 'str',
        'preference_name': 'str',
        'retina_display_setting': 'str'
    }

    attribute_map = {
        'id': 'id',
        'user_id': 'userId',
        'preference_name': 'preferenceName',
        'retina_display_setting': 'retinaDisplaySetting'
    }

    def __init__(self, id=None, user_id=None, preference_name=None, retina_display_setting=None):  # noqa: E501
        """BTUserWebPreferencesParams - a model defined in OpenAPI"""  # noqa: E501

        self._id = None
        self._user_id = None
        self._preference_name = None
        self._retina_display_setting = None
        self.discriminator = None

        if id is not None:
            self.id = id
        if user_id is not None:
            self.user_id = user_id
        if preference_name is not None:
            self.preference_name = preference_name
        if retina_display_setting is not None:
            self.retina_display_setting = retina_display_setting

    @property
    def id(self):
        """Gets the id of this BTUserWebPreferencesParams.  # noqa: E501


        :return: The id of this BTUserWebPreferencesParams.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this BTUserWebPreferencesParams.


        :param id: The id of this BTUserWebPreferencesParams.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def user_id(self):
        """Gets the user_id of this BTUserWebPreferencesParams.  # noqa: E501


        :return: The user_id of this BTUserWebPreferencesParams.  # noqa: E501
        :rtype: str
        """
        return self._user_id

    @user_id.setter
    def user_id(self, user_id):
        """Sets the user_id of this BTUserWebPreferencesParams.


        :param user_id: The user_id of this BTUserWebPreferencesParams.  # noqa: E501
        :type: str
        """

        self._user_id = user_id

    @property
    def preference_name(self):
        """Gets the preference_name of this BTUserWebPreferencesParams.  # noqa: E501


        :return: The preference_name of this BTUserWebPreferencesParams.  # noqa: E501
        :rtype: str
        """
        return self._preference_name

    @preference_name.setter
    def preference_name(self, preference_name):
        """Sets the preference_name of this BTUserWebPreferencesParams.


        :param preference_name: The preference_name of this BTUserWebPreferencesParams.  # noqa: E501
        :type: str
        """

        self._preference_name = preference_name

    @property
    def retina_display_setting(self):
        """Gets the retina_display_setting of this BTUserWebPreferencesParams.  # noqa: E501


        :return: The retina_display_setting of this BTUserWebPreferencesParams.  # noqa: E501
        :rtype: str
        """
        return self._retina_display_setting

    @retina_display_setting.setter
    def retina_display_setting(self, retina_display_setting):
        """Sets the retina_display_setting of this BTUserWebPreferencesParams.


        :param retina_display_setting: The retina_display_setting of this BTUserWebPreferencesParams.  # noqa: E501
        :type: str
        """

        self._retina_display_setting = retina_display_setting

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, BTUserWebPreferencesParams):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Onshape REST API

    The Onshape REST API consumed by all clients.  # noqa: E501

    OpenAPI spec version: 1.96
    Contact: api-support@onshape.zendesk.com
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class License(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'name': 'str',
        'url': 'str',
        'extensions': 'dict(str, object)'
    }

    attribute_map = {
        'name': 'name',
        'url': 'url',
        'extensions': 'extensions'
    }

    def __init__(self, name=None, url=None, extensions=None):  # noqa: E501
        """License - a model defined in OpenAPI"""  # noqa: E501

        self._name = None
        self._url = None
        self._extensions = None
        self.discriminator = None

        if name is not None:
            self.name = name
        if url is not None:
            self.url = url
        if extensions is not None:
            self.extensions = extensions

    @property
    def name(self):
        """Gets the name of this License.  # noqa: E501


        :return: The name of this License.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this License.


        :param name: The name of this License.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def url(self):
        """Gets the url of this License.  # noqa: E501


        :return: The url of this License.  # noqa: E501
        :rtype: str
        """
        return self._url

    @url.setter
    def url(self, url):
        """Sets the url of this License.


        :param url: The url of this License.  # noqa: E501
        :type: str
        """

        self._url = url

    @property
    def extensions(self):
        """Gets the extensions of this License.  # noqa: E501


        :return: The extensions of this License.  # noqa: E501
        :rtype: dict(str, object)
        """
        return self._extensions

    @extensions.setter
    def extensions(self, extensions):
        """Sets the extensions of this License.


        :param extensions: The extensions of this License.  # noqa: E501
        :type: dict(str, object)
        """

        self._extensions = extensions

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, License):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from onshape_client.configuration import Configuration
import onshape_client
from pathlib import Path
from ruamel.yaml import YAML
import os
from requests_oauthlib import OAuth2Session
from onshape_client.oauth.local_server import start_server
from enum import Enum
import webbrowser




class Client:
    """
    Initialize an Onshape client.

    The default is to pull the configuration from a file located in the user's home
    directory. However, configs can also be passed directly as a dictionary.

    Keys can be obtained from https://dev-portal.onshape.com.

    Parameters
    ----------
    conf_file : :obj:`str`, optional
        The path to the location of an onshape_client_config YAML file, where multiple stacks may be declared.
    configuration : :obj:`dict`, optional
        This overrides the conf_file parameter by supplying a dictionary that looks like the following:
        {
            "base_url": "https://cad.onshape.com",
            "access_key": "YOUR_ACCESS_KEY_HERE",
            "secret_key": "YOUR_SECRET_KEY_HERE"
        }
    stack_key : :obj:`str`, optional
        The key of the name for the particular stack to be used. Only applicable when reading configuration from the
        conf_file.
    """
    __instance = None

    @staticmethod
    def get_client():
        """ get an instance of the client class singleton. """
        if Client.__instance == None:
            raise Exception("Please manually instantiate the client.")
        return Client.__instance

    @staticmethod
    def get_configuration_from_keys_file(keys_file, stack_key):
        try:
            yaml = YAML()
            configurations_file = yaml.load(Path(keys_file))
            final_configuration = configurations_file[stack_key if stack_key else configurations_file['default_stack']]
        except KeyError as e:
            raise KeyError(
                "Your creds file is not constructed as expected. The key: {} was expected and now found.".format(e))
        return final_configuration

    def __init__(self, keys_file="~/.onshape_client_config.yaml", configuration=None, stack_key=None, open_authorize_grant_callback=None):
        """

        :param keys_file:
        :param configuration:
        :param stack_key:
        :param open_authorize_grant_callback: This will get called when the application needs to show the user the grant
            authorization page. It gets passed the URL to go to. Once the grant is authorized, the application needs to
            call client.set_grant_authorization_url_response(redirected_url) with whatever url the user got directed to
            (that includes the authorization code!)
        """
        if Client.__instance != None:
            raise Exception("This class is a singleton! Please use 'get_client' for all subsequent calls.")
        keys_file = os.path.expanduser(keys_file)
        if configuration:
            final_configuration = configuration
        elif keys_file:
            final_configuration = self.get_configuration_from_keys_file(keys_file, stack_key)
        else:
            raise EnvironmentError("API keys were not properly set.")

        self._set_configuration(final_configuration)
        self._create_apis()
        self.open_authorize_grant_callback = open_authorize_grant_callback

        if self.get_authentication_method() == "oauth":
            self._set_oauth_session()
            self.authorization_url, self.state = self.oauth.authorization_url(
                self.authorization_uri)

        Client.__instance = self

    def set_grant_authorization_url_response(self, authorization_url_response):
        """ This is the redirected-to url, for example: https:localhost/oauth_redirect?code=XraRXPYGSWfZmBvUIiNHvSlZ&state=kgZQJbVp731CgNxAeKJN7TIoOlo5bz"""
        self._fetch_access_token(authorization_url_response)

    def get_authentication_method(self):
        # Prefer OAUTH if specified, otherwise try for apikeys
        if self.client_secret and self.client_id:
            return "oauth"
        elif self.configuration.api_key['SECRET_KEY'] and self.configuration.api_key['ACCESS_KEY']:
            return "api_keys"
        else:
            return None

    def do_oauth_flow(self):
        """Do the oauth flow to set the access token"""
        try:
            self._refresh_access_token()
        except BaseException as e:
            if self.oauth_authorization_method == "localhost_server":
                start_server(self._fetch_access_token, self.open_authorize_grant)

    def open_authorize_grant(self):
        callback = self.open_authorize_grant_callback
        oauth_type = self.oauth_authorization_method
        url = self.authorization_url
        defaults_supported = ["localhost_server"]
        if not callback and oauth_type not in defaults_supported:
            raise NotImplementedError("To use OAuth, you need to pass in a callback to the client constructor with open_authorize_grant_callback=")
        elif callback:
            try:
                if callback and not callable(callback):
                    # Attempt to evaluate the function definition
                    callback = eval(callback)
                callback(url)
            except:
                raise AttributeError("The open_authorize_grant_callback function did not work.")
        else:
            self.default_callback(url)

    def default_callback(self, url):
        authorization_method = OAuthAuthorizationMethods(self.oauth_authorization_method)
        if authorization_method == OAuthAuthorizationMethods.LOCALHOST_SERVER:
            webbrowser.open(url)


    def _set_configuration(self, configuration_dictionary):
        configuration = Configuration()

        configuration.api_key['SECRET_KEY'] = self._get_if_present(configuration_dictionary, 'secret_key')
        configuration.api_key['ACCESS_KEY'] = self._get_if_present(configuration_dictionary, 'access_key')

        self.client_id = self._get_if_present(configuration_dictionary, 'client_id')
        self.client_secret = self._get_if_present(configuration_dictionary, 'client_secret')
        configuration.access_token = self._get_if_present(configuration_dictionary, 'access_token')
        self.refresh_token = self._get_if_present(configuration_dictionary, 'refresh_token')
        self.scope = self._get_if_present(configuration_dictionary, 'scope')
        self.redirect_uri = self._get_if_present(configuration_dictionary, 'redirect_uri')
        self.token_uri = self._get_if_present(configuration_dictionary, 'token_uri')
        self.authorization_uri = self._get_if_present(configuration_dictionary, 'authorization_url')
        self.scope = self._get_if_present(configuration_dictionary, 'scope')
        self.oauth_authorization_method = self._get_if_present(configuration_dictionary, 'oauth_authorization_method')

        configuration.host = self._get_if_present(configuration_dictionary, 'base_url')
        self.configuration = configuration
        return

    def _set_oauth_session(self):
        self.oauth = OAuth2Session(self.client_id, redirect_uri=self.redirect_uri,
                                           scope=self.scope)
        return

    def _refresh_access_token(self):
        # So that the oauth library doesn't complain about scope changing
        os.environ["OAUTHLIB_RELAX_TOKEN_SCOPE"] = "True"

        token_response = self.oauth.refresh_token(
            self.token_uri, refresh_token=self.refresh_token, client_id=self.client_id, client_secret=self.client_secret)
        self._set_oauth_creds_from_token_response(token_response)
        return

    def _fetch_access_token(self, authorization_response):
        try:
            token_response = self.oauth.fetch_token(
            self.token_uri,
            authorization_response=authorization_response,
            client_secret=self.client_secret)
            self._set_oauth_creds_from_token_response(token_response)
        except Warning:
            # This is probably a scope warning.
            pass
        return

    def _set_oauth_creds_from_token_response(self, token_response):
        self.configuration.access_token = token_response["access_token"]
        self.refresh_token = token_response["refresh_token"]
        return


    @staticmethod
    def _get_if_present(dictionary, key):
        return dictionary[key] if key in dictionary else ""

    def _create_apis(self):
        api_client = onshape_client.ApiClient(configuration=self.configuration)

        self.accounts_api = onshape_client.api.AccountsApi(api_client)
        self.app_elements_api = onshape_client.api.AppElementsApi(api_client)
        self.assemblies_api = onshape_client.api.AssembliesApi(api_client)
        self.blob_elements_api = onshape_client.api.BlobElementsApi(api_client)
        self.documents_api = onshape_client.api.DocumentsApi(api_client)
        self.elements_api = onshape_client.api.ElementsApi(api_client)
        self.endpoints_api = onshape_client.api.EndpointsApi(api_client)
        self.metadata_api = onshape_client.api.MetadataApi(api_client)
        self.parts_api = onshape_client.api.PartsApi(api_client)
        self.part_studios_api = onshape_client.api.PartStudiosApi(api_client)
        self.translation_api = onshape_client.api.TranslationsApi(api_client)

class OAuthAuthorizationMethods(Enum):
    LOCALHOST_SERVER="localhost_server"
    PYTHON_CALLBACK="python_callback"



# coding: utf-8

"""
    Onshape REST API

    The Onshape REST API consumed by all clients.  # noqa: E501

    OpenAPI spec version: 1.96
    Contact: api-support@onshape.zendesk.com
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class BTComponentUsagesSummary(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'hierarchy_': 'BTStandardContentHierarchy',
        'count': 'int',
        'hierarchy': 'BTStandardContentHierarchy'
    }

    attribute_map = {
        'hierarchy_': 'hierarchy_',
        'count': 'count',
        'hierarchy': 'hierarchy'
    }

    def __init__(self, hierarchy_=None, count=None, hierarchy=None):  # noqa: E501
        """BTComponentUsagesSummary - a model defined in OpenAPI"""  # noqa: E501

        self._hierarchy_ = None
        self._count = None
        self._hierarchy = None
        self.discriminator = None

        if hierarchy_ is not None:
            self.hierarchy_ = hierarchy_
        if count is not None:
            self.count = count
        if hierarchy is not None:
            self.hierarchy = hierarchy

    @property
    def hierarchy_(self):
        """Gets the hierarchy_ of this BTComponentUsagesSummary.  # noqa: E501


        :return: The hierarchy_ of this BTComponentUsagesSummary.  # noqa: E501
        :rtype: BTStandardContentHierarchy
        """
        return self._hierarchy_

    @hierarchy_.setter
    def hierarchy_(self, hierarchy_):
        """Sets the hierarchy_ of this BTComponentUsagesSummary.


        :param hierarchy_: The hierarchy_ of this BTComponentUsagesSummary.  # noqa: E501
        :type: BTStandardContentHierarchy
        """

        self._hierarchy_ = hierarchy_

    @property
    def count(self):
        """Gets the count of this BTComponentUsagesSummary.  # noqa: E501


        :return: The count of this BTComponentUsagesSummary.  # noqa: E501
        :rtype: int
        """
        return self._count

    @count.setter
    def count(self, count):
        """Sets the count of this BTComponentUsagesSummary.


        :param count: The count of this BTComponentUsagesSummary.  # noqa: E501
        :type: int
        """

        self._count = count

    @property
    def hierarchy(self):
        """Gets the hierarchy of this BTComponentUsagesSummary.  # noqa: E501


        :return: The hierarchy of this BTComponentUsagesSummary.  # noqa: E501
        :rtype: BTStandardContentHierarchy
        """
        return self._hierarchy

    @hierarchy.setter
    def hierarchy(self, hierarchy):
        """Sets the hierarchy of this BTComponentUsagesSummary.


        :param hierarchy: The hierarchy of this BTComponentUsagesSummary.  # noqa: E501
        :type: BTStandardContentHierarchy
        """

        self._hierarchy = hierarchy

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, BTComponentUsagesSummary):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Onshape REST API

    The Onshape REST API consumed by all clients.  # noqa: E501

    OpenAPI spec version: 1.96
    Contact: api-support@onshape.zendesk.com
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six


class BTBetaCapabilityMetricsInfo(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'state_pre_approved': 'int',
        'state_requested': 'int',
        'state_approved': 'int',
        'state_removed': 'int',
        'approved_pre_approved': 'int',
        'approved_requested': 'int',
        'approved_paid': 'int',
        'approved_onshape': 'int',
        'name': 'str',
        'id': 'str',
        'href': 'str',
        'view_ref': 'str'
    }

    attribute_map = {
        'state_pre_approved': 'statePreApproved',
        'state_requested': 'stateRequested',
        'state_approved': 'stateApproved',
        'state_removed': 'stateRemoved',
        'approved_pre_approved': 'approvedPreApproved',
        'approved_requested': 'approvedRequested',
        'approved_paid': 'approvedPaid',
        'approved_onshape': 'approvedOnshape',
        'name': 'name',
        'id': 'id',
        'href': 'href',
        'view_ref': 'viewRef'
    }

    def __init__(self, state_pre_approved=None, state_requested=None, state_approved=None, state_removed=None, approved_pre_approved=None, approved_requested=None, approved_paid=None, approved_onshape=None, name=None, id=None, href=None, view_ref=None):  # noqa: E501
        """BTBetaCapabilityMetricsInfo - a model defined in OpenAPI"""  # noqa: E501

        self._state_pre_approved = None
        self._state_requested = None
        self._state_approved = None
        self._state_removed = None
        self._approved_pre_approved = None
        self._approved_requested = None
        self._approved_paid = None
        self._approved_onshape = None
        self._name = None
        self._id = None
        self._href = None
        self._view_ref = None
        self.discriminator = None

        if state_pre_approved is not None:
            self.state_pre_approved = state_pre_approved
        if state_requested is not None:
            self.state_requested = state_requested
        if state_approved is not None:
            self.state_approved = state_approved
        if state_removed is not None:
            self.state_removed = state_removed
        if approved_pre_approved is not None:
            self.approved_pre_approved = approved_pre_approved
        if approved_requested is not None:
            self.approved_requested = approved_requested
        if approved_paid is not None:
            self.approved_paid = approved_paid
        if approved_onshape is not None:
            self.approved_onshape = approved_onshape
        if name is not None:
            self.name = name
        if id is not None:
            self.id = id
        if href is not None:
            self.href = href
        if view_ref is not None:
            self.view_ref = view_ref

    @property
    def state_pre_approved(self):
        """Gets the state_pre_approved of this BTBetaCapabilityMetricsInfo.  # noqa: E501


        :return: The state_pre_approved of this BTBetaCapabilityMetricsInfo.  # noqa: E501
        :rtype: int
        """
        return self._state_pre_approved

    @state_pre_approved.setter
    def state_pre_approved(self, state_pre_approved):
        """Sets the state_pre_approved of this BTBetaCapabilityMetricsInfo.


        :param state_pre_approved: The state_pre_approved of this BTBetaCapabilityMetricsInfo.  # noqa: E501
        :type: int
        """

        self._state_pre_approved = state_pre_approved

    @property
    def state_requested(self):
        """Gets the state_requested of this BTBetaCapabilityMetricsInfo.  # noqa: E501


        :return: The state_requested of this BTBetaCapabilityMetricsInfo.  # noqa: E501
        :rtype: int
        """
        return self._state_requested

    @state_requested.setter
    def state_requested(self, state_requested):
        """Sets the state_requested of this BTBetaCapabilityMetricsInfo.


        :param state_requested: The state_requested of this BTBetaCapabilityMetricsInfo.  # noqa: E501
        :type: int
        """

        self._state_requested = state_requested

    @property
    def state_approved(self):
        """Gets the state_approved of this BTBetaCapabilityMetricsInfo.  # noqa: E501


        :return: The state_approved of this BTBetaCapabilityMetricsInfo.  # noqa: E501
        :rtype: int
        """
        return self._state_approved

    @state_approved.setter
    def state_approved(self, state_approved):
        """Sets the state_approved of this BTBetaCapabilityMetricsInfo.


        :param state_approved: The state_approved of this BTBetaCapabilityMetricsInfo.  # noqa: E501
        :type: int
        """

        self._state_approved = state_approved

    @property
    def state_removed(self):
        """Gets the state_removed of this BTBetaCapabilityMetricsInfo.  # noqa: E501


        :return: The state_removed of this BTBetaCapabilityMetricsInfo.  # noqa: E501
        :rtype: int
        """
        return self._state_removed

    @state_removed.setter
    def state_removed(self, state_removed):
        """Sets the state_removed of this BTBetaCapabilityMetricsInfo.


        :param state_removed: The state_removed of this BTBetaCapabilityMetricsInfo.  # noqa: E501
        :type: int
        """

        self._state_removed = state_removed

    @property
    def approved_pre_approved(self):
        """Gets the approved_pre_approved of this BTBetaCapabilityMetricsInfo.  # noqa: E501


        :return: The approved_pre_approved of this BTBetaCapabilityMetricsInfo.  # noqa: E501
        :rtype: int
        """
        return self._approved_pre_approved

    @approved_pre_approved.setter
    def approved_pre_approved(self, approved_pre_approved):
        """Sets the approved_pre_approved of this BTBetaCapabilityMetricsInfo.


        :param approved_pre_approved: The approved_pre_approved of this BTBetaCapabilityMetricsInfo.  # noqa: E501
        :type: int
        """

        self._approved_pre_approved = approved_pre_approved

    @property
    def approved_requested(self):
        """Gets the approved_requested of this BTBetaCapabilityMetricsInfo.  # noqa: E501


        :return: The approved_requested of this BTBetaCapabilityMetricsInfo.  # noqa: E501
        :rtype: int
        """
        return self._approved_requested

    @approved_requested.setter
    def approved_requested(self, approved_requested):
        """Sets the approved_requested of this BTBetaCapabilityMetricsInfo.


        :param approved_requested: The approved_requested of this BTBetaCapabilityMetricsInfo.  # noqa: E501
        :type: int
        """

        self._approved_requested = approved_requested

    @property
    def approved_paid(self):
        """Gets the approved_paid of this BTBetaCapabilityMetricsInfo.  # noqa: E501


        :return: The approved_paid of this BTBetaCapabilityMetricsInfo.  # noqa: E501
        :rtype: int
        """
        return self._approved_paid

    @approved_paid.setter
    def approved_paid(self, approved_paid):
        """Sets the approved_paid of this BTBetaCapabilityMetricsInfo.


        :param approved_paid: The approved_paid of this BTBetaCapabilityMetricsInfo.  # noqa: E501
        :type: int
        """

        self._approved_paid = approved_paid

    @property
    def approved_onshape(self):
        """Gets the approved_onshape of this BTBetaCapabilityMetricsInfo.  # noqa: E501


        :return: The approved_onshape of this BTBetaCapabilityMetricsInfo.  # noqa: E501
        :rtype: int
        """
        return self._approved_onshape

    @approved_onshape.setter
    def approved_onshape(self, approved_onshape):
        """Sets the approved_onshape of this BTBetaCapabilityMetricsInfo.


        :param approved_onshape: The approved_onshape of this BTBetaCapabilityMetricsInfo.  # noqa: E501
        :type: int
        """

        self._approved_onshape = approved_onshape

    @property
    def name(self):
        """Gets the name of this BTBetaCapabilityMetricsInfo.  # noqa: E501


        :return: The name of this BTBetaCapabilityMetricsInfo.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this BTBetaCapabilityMetricsInfo.


        :param name: The name of this BTBetaCapabilityMetricsInfo.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def id(self):
        """Gets the id of this BTBetaCapabilityMetricsInfo.  # noqa: E501


        :return: The id of this BTBetaCapabilityMetricsInfo.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this BTBetaCapabilityMetricsInfo.


        :param id: The id of this BTBetaCapabilityMetricsInfo.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def href(self):
        """Gets the href of this BTBetaCapabilityMetricsInfo.  # noqa: E501


        :return: The href of this BTBetaCapabilityMetricsInfo.  # noqa: E501
        :rtype: str
        """
        return self._href

    @href.setter
    def href(self, href):
        """Sets the href of this BTBetaCapabilityMetricsInfo.


        :param href: The href of this BTBetaCapabilityMetricsInfo.  # noqa: E501
        :type: str
        """

        self._href = href

    @property
    def view_ref(self):
        """Gets the view_ref of this BTBetaCapabilityMetricsInfo.  # noqa: E501


        :return: The view_ref of this BTBetaCapabilityMetricsInfo.  # noqa: E501
        :rtype: str
        """
        return self._view_ref

    @view_ref.setter
    def view_ref(self, view_ref):
        """Sets the view_ref of this BTBetaCapabilityMetricsInfo.


        :param view_ref: The view_ref of this BTBetaCapabilityMetricsInfo.  # noqa: E501
        :type: str
        """

        self._view_ref = view_ref

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, BTBetaCapabilityMetricsInfo):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
